""" Terminal progressbars. """

import sys
import math
import time
import datetime
import collections

# Represents probablistic progress
# Example: tossing an unfair coin until we see heads
# `n`   number of tosses so far
# `n5`  number of tosses required such that on average 5% is done
# `n50` number of tosses required such that on average 50% is done
# `n95` number of tosses required such that on average 95% is done
# `p`   chance that we are done exactly after `n` tosses

prob_progress = collections.namedtuple('prob_progress',
                                ('n', 'n5', 'n50', 'n95', 'p'))

def coin(p, n):
    """ Returns prob_progress tuple for an unfair coin with chance p """
    cp = 1 - (1 - p) ** (n+1)
    n5 = math.log(1 - 0.05, 1 - p)
    n50 = math.log(1 - 0.5, 1 - p)
    n95 = math.log(1 - 0.95, 1 - p)
    return prob_progress(n, n5, n50, n95, cp)

class BaseProgressBar(object):
    """ Terminal progress bar accepting floats from the unit interval."""
    def __init__(self):
        raise NotImplementedError
    def __call__(self, value):
        raise NotImplementedError
    def start(self):
        raise NotImplementedError
    def end(self):
        raise NotImplementedError
    def __enter__(self):
        self.start()
        return self
    def __exit__(self, type, value, traceback):
        self.end()

class ProgressBar(BaseProgressBar):
    def  __init__(self):
        pass
    def __call__(self, value):
        width = 80 # TODO
        f = int(value * (width - 2))
        b = width - int(value * (width - 2)) - 2
        sys.stderr.write('\033[1G')
        sys.stderr.write('['+'='*f+' '*b+']')
        sys.stderr.flush()
    def start(self):
        pass
    def end(self):
        sys.stderr.write('\033[1G\n')
        sys.stderr.flush()

class ProbablisticProgressBar(BaseProgressBar):
    """ Terminal progress bar accepting prob_progress tuples """
    def __init__(self):
        pass
    def __call__(self, value):
        time_elapsed = time.time() - self.start_time
        speed = value.n / time_elapsed if time_elapsed else None
        t95 = int(max(value.n95 - value.n, 0) / speed) if speed else None
        t50 = int(max(value.n50 - value.n, 0) / speed) if speed else None
        t5 = int(max(value.n5 - value.n, 0) / speed) if speed else None
        width = 80 # TODO
        p95 = int(min(value.n / value.n95, 1) * (width-2))
        p50 = int(min(value.n / value.n50, 1) * (width-2))
        p5 = int(min(value.n / value.n5, 1) * (width-2))
        b95 = p95
        b50 = p50 - p95
        b5 = p5 - p50
        s = width - 2 - p5
        sys.stderr.write('\033[1G')
        sys.stderr.write('['+'#'*b95 + '='*b50 + '-'*b5+' '*s+']')
        sys.stderr.write('\n%5s tried, %5.1f/s %5.1f%%  %8s %8s %8s\033[1A' % (
                    value.n, speed, value.p*100,
                        datetime.timedelta(0, t5) if t5 else '',
                        datetime.timedelta(0, t50) if t50 else '',
                        datetime.timedelta(0, t95) if t95 else ''))
        sys.stderr.flush()
    def start(self):
        self.start_time = time.time()
    def end(self):
        sys.stderr.write('\033[1G\n\n')
        sys.stderr.flush()

if __name__ == '__main__':
    import time
    with ProgressBar() as p:
        for i in xrange(100):
            p(i/100.0)
            time.sleep(0.01)
    with ProbablisticProgressBar() as p:
        for i in xrange(100):
            p(coin(0.005, i))
            time.sleep(0.01)



import unittest
import functools

import pol.elgamal

import gmpy

class TestGroupParametersBase(unittest.TestCase):
    def _test_gp(self, gp, bits):
        q = (gp.p - 1) / 2
        self.assertTrue(gmpy.is_prime(gp.p))
        self.assertTrue(gmpy.is_prime(q))
        self.assertTrue(2**(bits-1) < gp.p)
        self.assertTrue(gp.p < 2**bits)
        self.assertTrue(gp.g < gp.p)
        self.assertTrue(3 < gp.g)
        self.assertNotEqual(pow(gp.g, 2, gp.p), 1)
        self.assertNotEqual(pow(gp.g, q, gp.p), 1)
        self.assertNotEqual(divmod(gp.p, gp.g)[1], 0)
        ginv = gmpy.invert(gp.g, gp.p)
        self.assertNotEqual(divmod(gp.p - 1, ginv)[1], 0)

class TestPrecomputedGroupParameters(TestGroupParametersBase):
    def _test_precomputed_parameters(self, bits):
        self._test_gp(pol.elgamal.precomputed_group_params(bits), bits)
for bits in (1025, 2049, 4097):
    def ch(bits): return lambda self: self._test_precomputed_parameters(bits)
    setattr(TestPrecomputedGroupParameters, 'test_%s' % bits, ch(bits))

class TestGeneratedGroupParameters(TestGroupParametersBase):
    def _test_generated_group_parameters(self, bits):
        gp = pol.elgamal.generate_group_params(bits)
        self._test_gp(gp, bits)
for bits in (128, 256, 512):
    def ch(bits):
        return lambda self: self._test_generated_group_parameters(bits)
    setattr(TestGeneratedGroupParameters, 'test_%s' % bits, ch(bits))

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

""" Entry point of the console application.

    Contains the argument parser and CLI interaction. """

# demandimport delays the import of modules until they are actually used.
import os
import sys
import demandimport

# FIXME the pyinstaller module loader breaks when running multiple processes
#       and demandimport is enabled.  Thus, when we are frozen by pyinstaller,
#       we disable demandimport.
if hasattr(sys, 'frozen'):
    import multiprocessing
elif 'POL_NO_DEMANDIMPORT' not in os.environ:
    demandimport.ignore('Crypto.PublicKey._fastmath')
    demandimport.enable()

import traceback
import readline
import argparse
import logging
import os.path
import getpass
import pprint
import shlex
import time
import math
import csv
import re
import os

import pol.safe
import pol.passgen
import pol.version
import pol.terminal
import pol.humanize
import pol.clipboard
import pol.progressbar

import pol.importers.keepass
import pol.importers.psafe3
import pol.speed

import msgpack
import yappi
import yaml

l = logging.getLogger(__name__)


# Used among others by `pol generate --hash-crack-time years'
cracktime_names = ('seconds', 'minutes', 'hours', 'days', 'months', 'years',
                   'decades', 'centuries', 'millennia', 'ages', 'astronomical')
cracktimes = {'seconds':     10,
              'minutes':     10*60,
              'hours':       10*60*60,
              'days':        10*60*60*24,
              'months':      10*60*60*24*30,
              'years':       10*60*60*24*365,
              'decades':     10*60*60*24*365*10,
              'centuries':   10*60*60*24*365*100,
              'millennia':   10*60*60*24*365*1000,
              'ages':        10*60*60*24*365*1000000,
              'astronomical':10*60*60*24*365*1000000000}

# TODO add commands
#   pol import
#       rename
#       regenerate
#       change-password

class Program(object):
    def __init__(self):
        # Contents of keyfiles, if provided
        self.additional_keys = None

    def parse_args(self, argv):
        """ Parse command line arguments.  Sets self.args. """
        # Common
        parser = argparse.ArgumentParser(add_help=False)
        g_basic = parser.add_argument_group('basic options')
        g_basic.add_argument('--safe', '-s', type=str,
                            metavar='PATH',
                    help='Path to safe')
        g_basic.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        g_basic.add_argument('--verbose', '-v', action='count',
                            dest='verbosity',
                    help='Add these to make pol chatty')
        g_basic.add_argument('--version', '-V', action=Program._VersionAction,
                            nargs=0,
                    help='Print version of pol and quit')
        g_advanced = parser.add_argument_group('advanced options')
        g_advanced.add_argument('--workers', '-w', type=int, metavar='N',
                    help='Number of workers processes (/threads)')
        g_advanced.add_argument('--threads', '-t', action='store_true',
                    help='Use worker threads instead of processes')
        g_advanced.add_argument('--profile', '-p', action='store_true',
                    help='Profile performance of main process')
        g_advanced.add_argument('--config-file', '-C', type=str, metavar='PATH',
                    help='Path to pol configuration file.')
        subparsers = parser.add_subparsers(title='commands')

        # pol init
        p_init = subparsers.add_parser('init', add_help=False,
                    help='Create a new safe')
        p_init_b = p_init.add_argument_group('basic options')
        p_init_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_init_b.add_argument('--force', '-f', action='store_true',
                    help='Remove any existing safe')
        p_init_a = p_init.add_argument_group('advanced options')
        p_init_a.add_argument('--rerand-bits', '-R', type=int, default=1025,
                    help='Minimal size in bits of prime used for '+
                            'rerandomization')
        p_init_a.add_argument('--precomputed-group-parameters', '-P',
                        action='store_true', dest='precomputed_gp',
                    help='Use precomputed group parameters for rerandomization')
        p_init_a.add_argument('--passwords', '-p', nargs='+', metavar='PW',
                    help='Passwords for containers as normally input '+
                            'interactively')
        p_init_a.add_argument('--i-know-its-unsafe', action='store_true',
                    help='Required for obviously unsafe actions')
        p_init_a.add_argument('--blocks', '-N', type=int, default=1024,
                    help='Number of blocks in the safe')
        p_init_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_init.set_defaults(func=self.cmd_init)

        # pol list
        p_list = subparsers.add_parser('list', add_help=False,
                    help='List entries')
        p_list_b = p_list.add_argument_group('basic options')
        p_list_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_list_b.add_argument('regex', nargs='?',
                    help='Only show entries with keys matching this regex')
        p_list_a = p_list.add_argument_group('basic options')
        p_list_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to list')
        p_list_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_list.set_defaults(func=self.cmd_list)

        # pol generate
        p_generate = subparsers.add_parser('generate', add_help=False,
                    help='Generate and store a password')
        p_generate.add_argument('key', nargs='?', default=None)
        p_generate_b = p_generate.add_argument_group('basic options')
        p_generate_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_generate_b.add_argument('note', nargs='?')
        p_generate_b.add_argument('--stdout', '-s', action='store_true',
                    help='write password to stdout')
        p_generate_b.add_argument('--no-copy', '-N', action='store_true',
                    help='Do not copy secret to clipboard.')
        p_generate_b.add_argument('--kind', '-k', default='dense',
                    choices=pol.passgen.kinds,
                    help='Kind of password to generate.')
        p_generate_s = p_generate_b.add_mutually_exclusive_group()
        p_generate_s.add_argument('--entropy', '-e', type=int, default=None,
                                metavar='N',
                    help='Desired strength of password')
        p_generate_s.add_argument('--length', '-l', type=int, default=None,
                                metavar='N',
                    help='Desired length of password')
        p_generate_s.add_argument('--web-crack-time', '-w',
                    choices=cracktime_names,
                    help=('Desired minimal time to brute force when '+
                            'allowed one try per second'))
        p_generate_s.add_argument('--hash-crack-time', '-H',
                    choices=cracktime_names,
                    help=('Desired minimal time to brute force when '+
                            'allowed a billion tries per second'))
        p_generate_s.add_argument('--sha-crack-cost', '-S', type=int,
                    metavar='DOLLARS',
                    help=('Desired minimal amount of dollars required to '+
                            'brute force if hashed with SHA-256'))
        p_generate_a = p_generate.add_argument_group('advanced options')
        p_generate_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to add password to')
        p_generate_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_generate.set_defaults(func=self.cmd_generate)

        # pol paste
        p_paste = subparsers.add_parser('paste', add_help=False,
                    help='Stores a secret from the clipboard')
        p_paste.add_argument('key')
        p_paste_b = p_paste.add_argument_group('basic options')
        p_paste_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_paste_b.add_argument('note', nargs='?')
        p_paste_a = p_paste.add_argument_group('advanced options')
        p_paste_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to add secret to')
        p_paste_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_paste.set_defaults(func=self.cmd_paste)

        # pol copy
        p_copy = subparsers.add_parser('copy', add_help=False,
                    help='Copies a password to the clipboard')
        p_copy.add_argument('key')
        p_copy_b = p_copy.add_argument_group('basic options')
        p_copy_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_copy_b.add_argument('-n', '--number', type=int, metavar='N',
                                default=None,
                    help='Pick, if multiple entries match, the Nth')
        p_copy_a = p_copy.add_argument_group('advanced options')
        p_copy_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to copy secret from')
        p_copy_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_copy.set_defaults(func=self.cmd_copy)

        # pol put
        p_put = subparsers.add_parser('put', add_help=False,
                    help='Stores a secret')
        p_put.add_argument('key')
        p_put_b = p_put.add_argument_group('basic options')
        p_put_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_put_b.add_argument('note', nargs='?')
        p_put_b.add_argument('--secret', '-s',
                    help='The secret to store.  If none is specified, reads '+
                         'secret from stdin.')
        p_put_a = p_put.add_argument_group('advanced options')
        p_put_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to add secret to')
        p_put_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_put.set_defaults(func=self.cmd_put)

        # pol get
        p_get = subparsers.add_parser('get', add_help=False,
                    help='Write secret to stdout')
        p_get.add_argument('key')
        p_get_b = p_get.add_argument_group('basic options')
        p_get_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_get_b.add_argument('-n', '--number', type=int, metavar='N',
                                default=None,
                    help='Pick, if multiple entries match, the Nth')
        p_get_a = p_get.add_argument_group('advanced options')
        p_get_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to get secret from')
        p_get_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_get.set_defaults(func=self.cmd_get)

        # pol remove
        p_remove = subparsers.add_parser('remove', add_help=False,
                    help='Removes an entry')
        p_remove.add_argument('key')
        p_remove_b = p_remove.add_argument_group('basic options')
        p_remove_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_remove_b.add_argument('-n', '--number', type=int, metavar='N',
                                default=None,
                    help='Pick, if multiple entries match, the Nth')
        p_remove_a = p_remove.add_argument_group('advanced options')
        p_remove_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to remove entry from')
        p_remove_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_remove.set_defaults(func=self.cmd_remove)

        # pol touch
        p_touch = subparsers.add_parser('touch', add_help=False,
                    help='Rerandomizes blocks')
        p_touch_b = p_touch.add_argument_group('basic options')
        p_touch_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_touch.set_defaults(func=self.cmd_touch)

        # pol raw
        p_raw = subparsers.add_parser('raw', add_help=False,
                    help='Shows raw data of safe')
        p_raw.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_raw.add_argument('--blocks', '-b', action='store_true',
                    help='Also print raw blocks')
        p_raw.add_argument('--passwords', '-p', nargs='+', metavar='PW',
                    help='Also show data of containers opened by '+
                            'these passwords')
        p_raw.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_raw.set_defaults(func=self.cmd_raw)

        # pol import-psafe3
        p_import_psafe3 = subparsers.add_parser('import-psafe3', add_help=False,
                    help='Imports entries from a psafe3 db')
        p_import_psafe3.add_argument('path',
                    help='Path to psafe3 database')
        p_import_psafe3_b = p_import_psafe3.add_argument_group(
                                    'basic options')
        p_import_psafe3_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_import_psafe3_a = p_import_psafe3.add_argument_group(
                                    'advanced options')
        p_import_psafe3_a.add_argument('-K', '--keyfiles', nargs='*',
                            metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_import_psafe3.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to import to')
        p_import_psafe3.add_argument('--psafe3-password', '-P',
                    metavar='PASSWORD',
                    help='Password of psafe3 db to import')
        p_import_psafe3.set_defaults(func=self.cmd_import_psafe3)

        # pol import-keepass
        p_import_keepass = subparsers.add_parser('import-keepass',
                        add_help=False,
                    help='Imports entries from a KeePass 1.x db')
        p_import_keepass.add_argument('path',
                    help='Path to KeePass database')
        p_import_keepass_b = p_import_keepass.add_argument_group(
                                    'basic options')
        p_import_keepass_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_import_keepass_b.add_argument('-k', '--keepass-keyfile',
                            metavar='PATH',
                    help='Keyfile used to open KeePass database')
        p_import_keepass_a = p_import_keepass.add_argument_group(
                                    'advanced options')
        p_import_keepass_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to import to')
        p_import_keepass_a.add_argument('--keepass-password', '-P',
                    metavar='PASSWORD',
                    help='Password of KeePass db to import')
        p_import_keepass_a.add_argument('-K', '--keyfiles', nargs='*',
                        metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_import_keepass.set_defaults(func=self.cmd_import_keepass)

        # pol export
        p_export = subparsers.add_parser('export',
                        add_help=False,
                    help='Exports entries to CSV')
        p_export.add_argument('path',
                    help='Path to CSV file to write to.  Defaults to stdout.',
                    default='-', nargs='?')
        p_export_b = p_export.add_argument_group('basic options')
        p_export_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_export_b.add_argument('-f', '--force', action='store_true',
                    help='Overwrite existing file')
        p_export_a = p_export.add_argument_group('advanced options')
        p_export_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to export')
        p_export_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_export.set_defaults(func=self.cmd_export)

        # pol import
        p_import = subparsers.add_parser('import',
                        add_help=False,
                    help='Imports entries from CSV')
        p_import.add_argument('path',
                    help='Path to CSV file to read.  Defaults to stdin.',
                    default='-', nargs='?')
        p_import_b = p_import.add_argument_group('basic options')
        p_import_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_import_a = p_import.add_argument_group('advanced options')
        p_import_a.add_argument('--password', '-p', metavar='PASSWORD',
                    help='Password of container to import to')
        p_import_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_import.set_defaults(func=self.cmd_import)

        # pol shell
        p_shell = subparsers.add_parser('shell',
                        add_help=False,
                    help='Start interactive shell')
        p_shell_b = p_shell.add_argument_group(
                                    'basic options')
        p_shell_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_shell_a = p_shell.add_argument_group('advanced options')
        p_shell_a.add_argument('-K', '--keyfiles', nargs='*', metavar='PATH',
                    help='Compose passwords with the contents of these files')
        p_shell.set_defaults(func=self.cmd_shell)

        # pol speed
        p_speed = subparsers.add_parser('speed',
                        add_help=False,
                    help='Measures speed of the components of pol')
        p_speed_b = p_speed.add_argument_group(
                                    'basic options')
        p_speed_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_speed.set_defaults(func=self.cmd_speed)

        p_nop = subparsers.add_parser('nop',
                        add_help=False,
                    help='Does nothing')
        p_nop_b = p_nop.add_argument_group(
                                    'basic options')
        p_nop_b.add_argument('-h', '--help', action='help',
                    help='show this help message and exit')
        p_nop_b.set_defaults(func=self.cmd_nop)

        self.args = parser.parse_args(argv)

    def load_configuration(self):
        """ Loads the configuration file, if present, and sets self.config """
        path = (self.args.config_file if self.args.config_file
                    else os.path.expanduser('~/.polrc'))
        if not os.path.exists(path):
            if self.args.config_file:
                sys.stderr.write("%s: no such file\n" % path)
                return -17
            l.debug('No configuration file found.')
            self.config = {}
            return
        cached_path = path + '.cached'
        if (os.path.exists(cached_path)
                    and os.stat(cached_path).st_mtime
                            > os.stat(path).st_mtime):
            l.debug('Loading cached configuration file %s ...', cached_path)
            with open(cached_path) as f:
                try:
                    self.config = msgpack.load(f)
                    l.debug('    ... done')
                    return
                except Exception as e:
                    l.warning('Exception loading cached configuration file.  '+
                                'Loading original.  (%s)', e)
        l.debug('Loading configuration file %s ...', path)
        with open(path) as f:
            try:
                self.config = yaml.load(f)
                if not self.config:
                    self.config = {}
            except yaml.YAMLError as e:
                sys.stderr.write("%s: error in configuration file:\n%s\n" % (
                                path, e))
                return -18
        l.debug('Writing cached configuration file %s ...', cached_path)
        with open(cached_path, 'w') as f:
            msgpack.dump(self.config, f)

    def main(self, argv):
        """ Main entry point. """
        try:
            profiling = False

            if 'POL_PROFILE' in os.environ:
                profiling = True
                yappi.start()

            if not argv:
                argv = ['shell']

            # Parse arguments
            self.parse_args(argv)

            # Profile?
            if self.args.profile and not profiling:
                profiling = True
                yappi.start()

            # Set up logging
            extra_logging_config = {}
            if self.args.verbosity >= 2:
                level = logging.DEBUG
                extra_logging_config['format'] = ('%(relativeCreated)d '+
                        '%(levelname)s %(name)s %(message)s')
            elif self.args.verbosity == 1:
                level = logging.INFO
            else:
                level = logging.WARNING
            logging.basicConfig(level=level, **extra_logging_config)

            # Load configuration
            ret = self.load_configuration()
            if ret:
                return ret

            # Set some global state
            self.safe_path = (self.args.safe if self.args.safe
                                else (self.config['safe']
                                        if 'safe' in self.config
                                    else os.path.expanduser('~/.pol')))
            self.keyfiles = (self.args.keyfiles if
                                    hasattr(self.args, 'keyfiles') and
                                            self.args.keyfiles
                                else (self.config['keyfiles']
                                        if 'keyfiles' in self.config
                                    else None))

            # Execute command
            ret = self._run_command()

            if profiling:
                yappi.stop()
                yappi.print_stats()

            return ret
        except Exception:
            self._handle_uncaught_exception()

    def cmd_init(self):
        if (os.path.exists(self.safe_path) and not self.args.force):
            print '%s exists.  Use -f to override.' % self.safe_path
            return -10
        self._ensure_keyfiles_are_loaded()
        if self.args.rerand_bits < 1025 and not self.args.i_know_its_unsafe:
            print 'You should now use less than 1025b group parameters.'
            return -9
        if self.args.precomputed_gp and not self.args.i_know_its_unsafe:
            # TODO are 2049 precomputed group parameters safe?
            print 'You should now use precomputed group parameters.'
            return -9
        if self.args.passwords:
            interactive = False
            cmdline_pws = list(reversed(self.args.passwords))
        else:
            interactive = True
        if interactive:
            print "You are about to create a new safe.  A safe can have up to six"
            print "separate containers to store your secrets.  A container is"
            print "accessed by one of its passwords.  Without one of its passwords,"
            print "you cannot prove the existence of a container."
            print
        first = True
        second = False
        pws = []
        for i in xrange(1, 7):
            if interactive:
                if not first:
                    print
                print 'Container #%s' % i
                if first:
                    print "  Each container must have a master-password.  This password gives"
                    print "  full access to the container."
                    print
                if second:
                    print "  Now enter the passwords for the second container."
                    print "  Leave blank if you do not want a second container."
                    print
            if interactive:
                if first:
                    masterpw = pol.terminal.zxcvbn_getpass(
                            'Enter master-password: ', '    ', False)
                else:
                    masterpw = pol.terminal.zxcvbn_getpass(
                            'Enter master-password [stop]: ', '    ')
            else:
                masterpw = cmdline_pws.pop() if cmdline_pws else ''
            if not masterpw:
                break
            if interactive and first:
                print
                print "  A container can have a list-password.  With this password you can"
                print "  list and add entries.  You cannot see the secrets of the existing"
                print "  entries.  Leave blank if you do not want a list-password."
                print
            if interactive:
                listpw = pol.terminal.zxcvbn_getpass(
                            'Enter list-password [no list-password]: ', '    ')
            else:
                listpw = cmdline_pws.pop() if cmdline_pws else ''
            if interactive and first:
                print
                print "  A container can have an append-password.  With this password you"
                print "  can only add entries.  You cannot see the existing entries."
                print "  Leave blank if you do not want an append-passowrd."
                print
            if interactive:
                appendpw = pol.terminal.zxcvbn_getpass(
                        'Enter append-password [no append-password]: ', '    ')
            else:
                appendpw = cmdline_pws.pop() if cmdline_pws else ''
            if second:
                second = False
            if first:
                first = False
                second = True
            pws.append((masterpw if masterpw else None,
                        listpw if listpw else None,
                        appendpw if appendpw else None))
        if interactive:
            print
        if not self.args.precomputed_gp:
            print 'Generating group parameters for this safe. This can take a while ...'
        # TODO generate group parameters in parallel
        progressbar = pol.progressbar.ProbablisticProgressBar()
        progressbar.start()
        def progress(step, x):
            if step == 'p' and x is None:
                progressbar.start()
            elif step == 'p' and x:
                progressbar(x)
            elif step == 'g':
                progressbar.end()
        try:
            blocks_per_container = int(math.floor(self.args.blocks / 6.0))
            with pol.safe.create(os.path.expanduser(self.safe_path),
                                 override=self.args.force,
                                 nworkers=self.args.workers,
                                 gp_bits=self.args.rerand_bits,
                                 progress=progress,
                                 precomputed_gp=self.args.precomputed_gp,
                                 use_threads=self.args.threads,
                                 n_blocks=self.args.blocks) as safe:
                for i, mlapw in enumerate(pws):
                    mpw, lpw, apw = mlapw
                    print '  allocating container #%s ...' % (i+1)
                    c = safe.new_container(mpw, lpw, apw,
                                    additional_keys=self.additional_keys,
                                    nblocks=blocks_per_container)
                print '  trashing freespace ...'
                safe.trash_freespace()
        except pol.safe.SafeAlreadyExistsError:
            print '%s exists.  Use -f to override.' % self.safe_path
            return -10
    
    def cmd_touch(self):
        with self._open_safe() as safe:
            safe.touch()

    def cmd_raw(self):
        with self._open_safe() as safe:
            d = dict(safe.data)
            if not self.args.blocks:
                del d['blocks']
            pprint.pprint(d)
            if not self.args.passwords:
                return
            for password in self.args.passwords:
                for container in self._open_containers(safe, password):
                    print
                    print 'Container %s' % container.id
                    if container.main_data:
                        pprint.pprint(container.main_data)
                    if container.append_data:
                        pprint.pprint(container.append_data)
                    if container.secret_data:
                        pprint.pprint(container.secret_data)

    def cmd_get(self):
        with self._open_safe() as safe:
            found_one = False
            entries = []
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter password: ')):
                if not found_one:
                    found_one = True
                try:
                    for entry in container.get(self.args.key):
                        if not entry.has_secret:
                            continue
                        entries.append((container, entry))
                except pol.safe.MissingKey:
                    continue
                except KeyError:
                    continue
            if not found_one:
                sys.stderr.write('The password did not open any container.\n')
                return -1
            if not entries:
                sys.stderr.write('No entries found.\n')
                return -4
            if len(entries) > 1 and not self.args.number:
                sys.stderr.write('Multiple entries found:\n')
                sys.stderr.write('\n')
                for n, container_entry in enumerate(entries):
                    container, entry = container_entry
                    sys.stderr.write(' %2s. %-20s %s\n' % (n+1, entry.key,
                            repr(entry.note) if entry.note else ''))
                sys.stderr.write('\n')
                sys.stderr.write('Use `-n N\' to pick one.\n')
                return -8
            n = self.args.number - 1 if self.args.number else 0
            if n < 0 or n >= len(entries):
                sys.stderr.write('Entry number out of range.\n')
                return -15
            entry = entries[n][1]
            sys.stderr.write(' note: %s\n' % repr(entry.note))
            print entry.secret

    def cmd_remove(self):
        with self._open_safe() as safe:
            found_one = False
            entries = []
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter password: ')):
                if not found_one:
                    found_one = True
                try:
                    for entry in container.get(self.args.key):
                        if not entry.has_secret:
                            continue
                        entries.append((container, entry))
                except pol.safe.MissingKey:
                    continue
                except KeyError:
                    continue
            if not found_one:
                sys.stderr.write('The password did not open any container.\n')
                return -1
            if not entries:
                sys.stderr.write('No entries found.\n')
                return -4
            if len(entries) > 1 and not self.args.number:
                sys.stderr.write('Multiple entries found:\n')
                sys.stderr.write('\n')
                for n, container_entry in enumerate(entries):
                    container, entry = container_entry
                    sys.stderr.write(' %2s. %-20s %s\n' % (n+1, entry.key,
                            repr(entry.note) if entry.note else ''))
                sys.stderr.write('\n')
                sys.stderr.write('Use `-n N\' to pick one.\n')
                return -8
            n = self.args.number - 1 if self.args.number else 0
            if n < 0 or n >= len(entries):
                sys.stderr.write('Entry number out of range.\n')
                return -15
            entries[n][1].remove()

    def cmd_copy(self):
        if not pol.clipboard.available:
            print 'Clipboard access not available.'
            print 'Use `pol get\' to print secrets.'
            return -7
        with self._open_safe() as safe:
            found_one = False
            entries = []
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter password: ')):
                if not found_one:
                    found_one = True
                try:
                    for entry in container.get(self.args.key):
                        if not entry.has_secret:
                            continue
                        entries.append((container, entry))
                except pol.safe.MissingKey:
                    continue
                except KeyError:
                    continue
            if not found_one:
                print 'The password did not open any container.'
                return -1
            if not entries:
                print 'No entries found'
                return -4
            if len(entries) > 1 and not self.args.number:
                sys.stderr.write('Multiple entries found:\n')
                sys.stderr.write('\n')
                for n, container_entry in enumerate(entries):
                    container, entry = container_entry
                    sys.stderr.write(' %2s. %-20s %s\n' % (n+1, entry.key,
                            repr(entry.note) if entry.note else ''))
                sys.stderr.write('\n')
                sys.stderr.write('Use `-n N\' to pick one.\n')
                return -8
            n = self.args.number - 1 if self.args.number else 0
            if n < 0 or n >= len(entries):
                sys.stderr.write('Entry number out of range.\n')
                return -15
            entry = entries[n][1]
            print ' note: %s' % repr(entry.note)
            print 'Copied secret to clipboard.  Press any key to clear ...'
            pol.clipboard.copy(entry.secret)
            pol.terminal.wait_for_keypress()
            pol.clipboard.clear()
    def cmd_paste(self):
        if not pol.clipboard.available:
            print 'Clipboard access not available.'
            print 'Use `pol put\' to add passwords from stdin.'
            return -7
        pw = pol.clipboard.paste()
        if not pw:
            print 'Clipboard is empty'
            return -3
        return self._store(pw)
        pol.clipboard.clear()
    def cmd_put(self):
        pw = self.args.secret if self.args.secret else sys.stdin.read()
        if not pw:
            print 'No secret given'
            return -3
        return self._store(pw)
    def _store(self, pw):
        """ Common code of `pol put', `pol generate' and `pol paste' -
            stores `pw' to an entry self.args.key. """
        with self._open_safe() as safe:
            found_one = False
            stored = False
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter (append-)password: ')):
                if not found_one:
                    found_one = True
                try:
                    container.add(self.args.key, self.args.note, pw)
                    container.save()
                    stored = True
                    break
                except pol.safe.MissingKey:
                    pass
            if not found_one:
                print 'The password did not open any container.'
                return -1
            if found_one and not stored:
                print 'No append access to the containers opened by this password'
                return -2
    def cmd_generate(self):
        if self.args.hash_crack_time:
            self.args.entropy = math.log(1000000000 * cracktimes[
                                        self.args.hash_crack_time], 2)
        if self.args.web_crack_time:
            self.args.entropy = math.log(cracktimes[
                                        self.args.web_crack_time], 2)
        if self.args.sha_crack_cost:
            self.args.entropy = math.log(self.args.sha_crack_cost
                                            * 500000000, 2)
            # We use the bitcoin mining rate as an estimator
            #   http://blockchain.info/stats
            # TODO keep up-to-date
        pw = pol.passgen.generate_password(length=self.args.length,
                                           entropy=self.args.entropy,
                                           kind=self.args.kind)
        if self.args.key is None:
            if self.args.stdout:
                sys.stdout.write(pw)
                sys.stdout.write("\n")
                return
            if self.args.no_copy:
                return
            if not pol.clipboard.available:
                sys.stderr.write('Clipboard access not available.\n')
                sys.stderr.write('Use --stdout to write to print password instead.\n')
                return -7
            pol.clipboard.copy(pw)
            sys.stderr.write('Copied password to clipboard.  '+
                                'Press any key to clear ...\n')
            pol.terminal.wait_for_keypress()
            pol.clipboard.clear()
            return
        found_one = False
        stored = False
        with self._open_safe() as safe:
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter (append-)password: ')):
                if not found_one:
                    found_one = True
                try:
                    container.add(self.args.key, self.args.note, pw)
                    container.save()
                    stored = True
                    break
                except pol.safe.MissingKey:
                    pass
            if not found_one:
                sys.stderr.write('The password did not open any container.\n')
                return -1
            if found_one and not stored:
                sys.stderr.write('No append access to the containers opened '+
                                        'by this password\n')
                return -2
            if self.args.stdout:
                sys.stdout.write(pw)
                sys.stdout.write("\n")
                return
            if self.args.no_copy:
                return
            if not pol.clipboard.available:
                sys.stderr.write('Password stored.  Clipboard access not available.\n')
                sys.stderr.write('Use `pol get\' to show password\n')
                return
            pol.clipboard.copy(pw)
            sys.stderr.write('Copied password to clipboard.  Press any key to clear ...\n')
            pol.terminal.wait_for_keypress()
            pol.clipboard.clear()
            # TODO do rerandomization in parallel

    def cmd_list(self):
        if self.args.regex:
            try:
                regex = re.compile(self.args.regex, re.I)
            except re.error as e:
                sys.stderr.write("Invalid regex: %s\n" % e.message)
                return -16
        else:
            regex = None
        with self._open_safe() as safe:
            found_one = False
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter (list-)password: ')):
                if not found_one:
                    found_one = True
                else:
                    print
                print 'Container @%s' % container.id
                try:
                    got_entry = False
                    for entry in container.list():
                        if regex and not regex.search(entry.key):
                            continue
                        got_entry = True
                        print ' %-20s %s' % (entry.key, repr(entry.note)
                                                    if entry.note else '')
                    if not got_entry:
                        if regex:
                            print '  (no matching entries)'
                        else:
                            print '  (empty)'
                except pol.safe.MissingKey:
                    print '  (no list access)'
            if not found_one:
                print ' No containers found'

    def cmd_import(self):
        entries = []
        close_f = False
        # First, read the CSV
        if self.args.path == '-':
            f = sys.stdin
        else:
            if not os.path.exists(self.args.path):
                sys.stderr.write("%s: no such file\n" % self.args.path)
                return -19
            f = open(self.args.path)
            close_f = True
        try:
            reader = csv.reader(f)
            for row in reader:
                if len(row) != 3:
                    sys.stderr.write("%s: row should have exactly 3 entries\n"
                                        % self.args.path)
                    return -20
                entries.append(row)
        finally:
            if close_f:
                f.close()

        # Then, open the pol safe
        with self._open_safe() as safe:
            found_one = False
            the_container = None
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter (append-)password: ')):
                if not found_one:
                    found_one = True
                if container.can_add:
                    the_container = container
                    break
            if not found_one:
                print 'The password did not open any container.'
                return -1
            if not the_container:
                print ('No append access to the containers opened '+
                            'by this password')
                return -2

            # Import the entries
            for entry in entries:
                the_container.add(*entry)
            the_container.save()
            print "%s entries imported" % len(entries)

    def cmd_import_keepass(self):
        # First load keepass db
        kppwd = (self.args.keepass_password if self.args.keepass_password
                        else getpass.getpass('Enter password for KeePass db: '))
        fkeyfile = None
        if self.args.keepass_keyfile:
            fkeyfile = open(self.args.keepass_keyfile)
        try:
            with open(self.args.path) as f:
                groups, entries = pol.importers.keepass.load(f, kppwd, fkeyfile)
        finally:
            if fkeyfile:
                fkeyfile.close()

        # Secondly, find a container
        with self._open_safe() as safe:
            found_one = False
            the_container = None
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                        else getpass.getpass('Enter (append-)password: ')):
                if not found_one:
                    found_one = True
                if container.can_add:
                    the_container = container
                    break
            if not found_one:
                print 'The password did not open any container.'
                return -1
            if not the_container:
                print ('No append access to the containers opened '+
                            'by this password')
                return -2

            # Import the entries
            n_imported = 0
            for entry in entries:
                if not entry['uuid'].int:
                    continue
                notes = []
                n_imported += 1
                if 'notes' in entry and entry['notes']:
                    notes.append(entry['notes'])
                if 'username' in entry and entry['username']:
                    notes.append('user: '+entry['username'])
                if 'url' in entry and entry['url']:
                    notes.append('url: '+entry['url'])
                the_container.add(entry['title'],
                                  '\n'.join(notes),
                                  entry['password'])
            the_container.save()
            print "%s entries imported" % n_imported

    def cmd_import_psafe3(self):
        # First load psafe3 db
        ps3pwd = (self.args.psafe3_password if self.args.psafe3_password
                        else getpass.getpass('Enter password for psafe3 db: '))
        with open(self.args.path) as f:
            header, records = pol.importers.psafe3.load(f, ps3pwd)

        # Secondly, find a container
        with self._open_safe() as safe:
            found_one = False
            the_container = None
            for container in self._open_containers(safe,
                    self.args.password if self.args.password
                            else getpass.getpass('Enter (append-)password: ')):
                if not found_one:
                    found_one = True
                if container.can_add:
                    the_container = container
                    break
            if not found_one:
                print 'The password did not open any container.'
                return -1
            if not the_container:
                print ('No append access to the containers opened '+
                            'by this password')
                return -2

            # Import the records
            for record in records:
                notes = []
                if 'notes' in record and record['notes']:
                    notes.append(record['notes'])
                if 'email-address' in record and record['email-address']:
                    notes.append('email: '+record['email-address'])
                if 'username' in record and record['username']:
                    notes.append('user: '+record['username'])
                if 'url' in record and record['url']:
                    notes.append('url: '+record['url'])
                the_container.add(record['title'],
                                  '\n'.join(notes),
                                  record['password'])
            the_container.save()
            print "%s records imported" % len(records)

    def cmd_speed(self):
        return pol.speed.main(self)

    def cmd_shell(self):
        with demandimport.disabled():
            import readline
        # TODO a more stateful shell would be nice: then we only have to
        #       ask for the password and rerandomize once.
        if not os.path.exists(self.safe_path):
            print "No safe found.  Type `init' to create a new safe."
        while True:
            try:
                line = raw_input('pol> ').strip()
            except EOFError:
                sys.stderr.write("\n")
                break
            except KeyboardInterrupt:
                sys.stderr.write("\nUse C-d to quit.\n")
                continue
            if not line:
                continue
            argv = shlex.split(line)
            try:
                self.parse_args(argv)
            except SystemExit:
                continue
            if self.args.func == self.cmd_shell:
                continue
            self._run_command()

    def cmd_export(self):
        close_f = False
        rows_written = 0
        found_one = False
        try:
            if self.args.path == '-':
                f = sys.stdout
            else:
                if os.path.exists(self.args.path) and not self.args.force:
                    sys.stderr.write("%s exists. Use -f to override.\n"
                                            % self.args.path)
                    return -11
                f = open(self.args.path, 'w')
                close_f = True
            writer = csv.writer(f)
            with self._open_safe() as safe:
                for container in self._open_containers(safe,
                        self.args.password if self.args.password
                                else getpass.getpass('Enter password: ')):
                    found_one = True
                    for entry in container.list():
                        rows_written += 1
                        writer.writerow([entry.key, entry.note, entry.secret])
            if not found_one:
                sys.stderr.write("The password did not open any container.\n")
                return -1
        finally:
            if close_f:
                f.close()
        sys.stderr.write("%s entries exported.\n" % rows_written)

    def cmd_nop(self):
        pass

    def _on_move_append_entries(self, entries):
        """ Called when entries entered by an append-only-password are moved
            into the container. """
        sys.stderr.write("  moved entries into container: %s\n" % (
                pol.humanize.join([entry[0] for entry in entries])))
    def _open_safe(self):
        return pol.safe.open(os.path.expanduser(self.safe_path),
                           nworkers=self.args.workers,
                           use_threads=self.args.threads,
                           progress=Program._RerandProgress())
    def _run_command(self):
        try:
            return self.args.func()
        except pol.safe.SafeNotFoundError:
            sys.stderr.write("%s: no such file.\n" % self.safe_path)
            sys.stderr.write("To create a new safe, run `pol init'.\n")
            return -5
        except pol.safe.SafeLocked:
            sys.stderr.write("%s: locked.\n" % self.safe_path)
            # TODO add a `pol break-lock'
            return -6
        except pol.safe.WrongMagicError:
            sys.stderr.write("%s: not a pol safe.\n" % self.safe_path)
            return -13
        except KeyboardInterrupt:
            sys.stderr.write("\n^C\n")
            return -14
        except Exception:
            self._handle_uncaught_exception()
            return -12
        # TODO gracefully handle SafeFullError

    def _ensure_keyfiles_are_loaded(self):
        """ Ensures self.additional_keys is set to the contents of the
            desired keyfiles. """
        if self.additional_keys or not self.keyfiles:
            return
        self.additional_keys = []
        l.debug('Loading keyfiles ...')
        for keyfile in self.keyfiles:
            l.debug('  %s ...', keyfile)
            with open(keyfile) as f:
                self.additional_keys.append(f.read())

    def _open_containers(self, safe, password):
        self._ensure_keyfiles_are_loaded()
        return safe.open_containers(password,
                        on_move_append_entries=self._on_move_append_entries,
                        additional_keys=self.additional_keys)

    def _handle_uncaught_exception(self):
        sys.stderr.write("\n")
        sys.stderr.write("An unhandled exception occured:\n")
        sys.stderr.write("\n   ")
        sys.stderr.write(traceback.format_exc().replace("\n", "\n   "))
        sys.stderr.write("\n")
        sys.stderr.write("Please report this error:\n")
        sys.stderr.write("\n")
        sys.stderr.write("   https://github.com/bwesterb/pol/issues\n")
        sys.stderr.write("\n")
        sys.stderr.flush()

    class _RerandProgress():
        """ Glue between callbacks of rerandomize and the progressbar. """
        def __init__(self):
            self.progressbar = pol.progressbar.ProgressBar()
            self.started = False
            self.starting_time = None
        def __call__(self, v):
            if self.starting_time is None:
                self.starting_time = time.time()
            if not self.started and time.time() - self.starting_time > 1.0:
                self.started = True
                self.progressbar.start()
            if not self.started:
                return
            self.progressbar(v)
            if v == 1.0:
                self.progressbar.end()

    class _VersionAction(argparse.Action):
        def __call__(self, parser, namespace, values, option_stirng):
            print pol.version.get_version()
            sys.exit()


def entrypoint(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    return Program().main(argv)

if __name__ == '__main__':
    sys.exit(entrypoint())



""" Extensions to Python's multiprocessing. """

import time
import Queue
import threading
import multiprocessing

def parallel_map(func, seq, args=None, kwargs=None, chunk_size=1,
                        nworkers=None, progress=None, progress_interval=0.1,
                        initializer=None, use_threads=False):
    """ Similar to map, but executes in parallel.

            parallel_map(f, seq, args, kwargs)

        gives the same result as

            [f(x, *args, **kwargs) for x in seq]

            `nworkers`  number of workers to spawn
            `chunk_size`    number of elements to hand to a thread at the
                        same time.
            `progress`  a function to periodically call with the number of
                        elements of seq already mapped
            `progress_interval`     the approximate time interval to call
                        the `progress` function
            `initializer`           called in each worker process when spawn,
                        with (args, kwargs) as arguments.  `initializer`
                        may change args, kwargs.
            `use_threads`   specifies to use threads instead of processes. """
    # Shortcut for when there is only one chunk:
    if args is None:
        args = ()
    if kwargs is None:
        kwargs = {}
    if len(seq) <= chunk_size:
        if initializer is not None:
            initializer(args, kwargs)
        return  [func(x, *args, **kwargs) for x in seq]
    # We got more than one chunk --- we will need workers:
    def worker(c_func, c_args, c_kwargs, c_input, c_output, c_initializer):
        try:
            if c_initializer is not None:
                c_initializer(c_args, c_kwargs)
            while True:
                p = c_input.get()
                if p is None:
                    break
                i, xs = p
                ys = []
                for x in xs:
                    ys.append(c_func(x, *c_args, **c_kwargs))
                c_output.put((i, ys))
        except KeyboardInterrupt:
            pass
    if nworkers is None:
        nworkers = multiprocessing.cpu_count()
    p_input = multiprocessing.Queue()
    p_output = multiprocessing.Queue()
    processes = []
    N = len(seq)
    n = 0
    ret = [None]*N
    constr = threading.Thread if use_threads else multiprocessing.Process
    try:
        for i in xrange(nworkers):
            process = constr(target=worker, args=(func, args, kwargs, p_output,
                                                    p_input, initializer))
            processes.append(process)
            process.start()
        # Add the elements to be mapped to the queue
        for i in xrange(0, N, chunk_size):
            p_output.put((i, seq[i:i+chunk_size]))
        # and after that sentinels to signal the end
        # of the queue, one for each worker
        for i in xrange(nworkers):
            p_output.put(None)
        next_update = (time.time() + progress_interval
                            if progress else float('inf'))
        while n < N:
            i, ys = p_input.get()
            ret[i:i+len(ys)] = ys
            n += len(ys)
            if time.time() > next_update:
                next_update = time.time() + progress_interval
                progress(n)
    except KeyboardInterrupt:
        for process in processes:
            process.terminate()
        raise
    return ret

def parallel_try(func, args=None, kwargs=None, nworkers=None, progress=None,
                        progress_interval=0.1, update_interval=0.05,
                        initializer=None, join=True, use_threads=False):
    """ Execute `func` in parallel until it returns a value.
        Return that value.
        
            `func`      the function to call
            `args`      the arguments to `func`
            `kwargs`    the keyword arguments to `func`
            `nworkers`  the number of worker processes (/threads) to use
            `progress`  a function to periodically call with the number of
                        tries already performed
            `progress_interval`     the approximate time interval to call
                        the `progress` function
            `update_interval`       the approximate interval in which the
                        worker threads report their status --- a higher
                        values gives better performance, but worse latency
                        (if `join` is True)
            `initializer`           called in each worker process when spawn,
                        with (args, kwargs) as arguments.  `initializer`
                        may change args, kwargs.
            `join`      specifies whether to wait for all threads to finish
            `use_threads`   specifies to use threads instead of processes. """
    def worker(c_func, c_args, c_kwargs, c_lock, c_done, c_output, c_counter,
                        c_initializer):
        try:
            if c_initializer is not None:
                c_initializer(c_args, c_kwargs)
            last_update = time.time()
            iterations = 0
            while True:
                now = time.time()
                if now - last_update >  update_interval:
                    last_update = now
                    with c_lock:
                        if c_done.is_set():
                            return
                        c_counter.value += iterations
                    iterations = 0
                ret = c_func(*c_args, **c_kwargs)
                iterations += 1
                if ret is None:
                    continue
                with c_lock:
                    if c_done.is_set():
                        return
                    c_output.put(ret)
                    c_done.set()
        except KeyboardInterrupt:
            pass
    if args is None:
        args = ()
    if kwargs is None:
        kwargs = {}
    if nworkers is None:
        nworkers = multiprocessing.cpu_count()
    p_done = multiprocessing.Event()
    p_lock = multiprocessing.Lock()
    p_counter = multiprocessing.RawValue('i', 0)
    p_input = multiprocessing.Queue()
    processes = []
    constr = threading.Thread if use_threads else multiprocessing.Process
    try:
        for i in xrange(nworkers):
            process = constr(target=worker, args=(func, args, kwargs, p_lock,
                                        p_done, p_input, p_counter,
                                        initializer))
            processes.append(process)
            process.start()
        if progress is None:
            p_done.wait()
        else:
            while not p_done.is_set():
                p_done.wait(progress_interval)
                progress(p_counter.value)
        if join:
            for process in processes:
                process.join()
    except KeyboardInterrupt:
        for process in processes:
            process.terminate()
        raise
    return p_input.get()



""" Implementation of key stretching  """

import logging

import Crypto.Random
import scrypt

# FIXME There is a bug in scrypt 0.5.5 that causes a crash when we
#       try to specify r or p.

l = logging.getLogger(__name__)

class KeyStretchingParameterError(ValueError):
    pass

class KeyStretching(object):
    """ Derives a key from a password. """

    def __init__(self, params):
        """ Initialize the KeyStretching with the given parameters.

            NOTE use KeyStretching.setup """
        self.params = params

    @staticmethod
    def setup(params=None, randfunc=None):
        """ Set-up the key-stretching given by `params`.

            If `params' is None, generates new parameters.  In that case
            `randfunc' is used to generate a salt. """
        if params is None:
            if randfunc is None:
                randfunc = Crypto.Random.new().read
            params = {'type': 'scrypt',
                      'salt': randfunc(32),
                      #'r': 8,
                      #'p': 1,
                      'Nexp': 15}
        if ('type' not in params or not isinstance(params['type'], basestring)
                or params['type'] not in TYPE_MAP):
            raise KeyStretchingParameterError("Invalid `type' attribute")
        return TYPE_MAP[params['type']](params)

    def __call__(self, password):
        return self.stretch(password)

    def stretch(self, password):
        raise NotImplementedError

class ScryptKeyStretching(KeyStretching):
    """ scrypt is the default for key stretching """

    def __init__(self, params):
        super(ScryptKeyStretching, self).__init__(params)
        #for attr in ('Nexp', 'r', 'p'):
        for attr in ('Nexp',):
            if not attr in params:
                raise KeyStretchingParameterError("Missing param `%s'" % attr)
            if not isinstance(params[attr], int):
                raise KeyStretchingParameterError("%s should be int" % attr)
            if params[attr] < 0:
                raise KeyStretchingParameterError("%s should be positive"% attr)
        if not 'salt' in params or not isinstance(params['salt'], basestring):
            raise KeyStretchingParameterError("Invalid param `salt'")
        #if params['r'] * params['p'] >= 2**30:
        #    raise KeyStretchingParameterError("r*p is too large")
        if params['Nexp'] <= 1:
            raise KeyStretchingParameterError("Nexp is too small")

    def stretch(self, password):
        return scrypt.hash(password,
                           self.params['salt'],
                           #r=self.params['r'],
                           #p=self.params['p'],
                           N=2**self.params['Nexp'])

TYPE_MAP = {'scrypt': ScryptKeyStretching}



import unittest
import binascii

import pol.kd

class TestSHA(unittest.TestCase):
    def setUp(self):
        self.kd = pol.kd.KeyDerivation.setup()
    def test_default(self):
        self.assertEqual(self.kd.params['type'], 'sha')
    def test_restore(self):
        self.kd2 = pol.kd.KeyDerivation.setup(self.kd.params)
        self.assertEqual(self.kd2.params['salt'], self.kd.params['salt'])
        self.assertEqual(self.kd2(['abc']), self.kd(['abc']))
    def test_single_unsalted(self):
        kd = pol.kd.KeyDerivation.setup({'bits': 256, 'type': 'sha', 'salt':''})
        self.assertEqual(binascii.hexlify(kd(['abc'])),
                        '4be9b856664b17f617ac8f072c48f80f'+
                        '6b008abb9f0499c2a5914b5683f3386f')
    def test_single(self):
        kd = pol.kd.KeyDerivation.setup({'bits': 256, 'type': 'sha','salt':'c'})
        self.assertEqual(binascii.hexlify(kd(['ab'])),
                        '530c455f8ff55d3d40bac27074c8f730'+
                        '8c7bd40e03808353e3296207ada91388')
    def test_multiple_salted(self):
        kd = pol.kd.KeyDerivation.setup({'bits': 256, 'type': 'sha','salt':'c'})
        self.assertEqual(binascii.hexlify(kd(['a', 'b', 'c'])),
                        'b87a32912e780ab8e22555d132fec8c0'+
                        '1b2867128ebb4e56dcac029e71ac902f')
    def test_multiple_unsalted(self):
        kd = pol.kd.KeyDerivation.setup({'bits': 256, 'type': 'sha','salt': ''})
        self.assertEqual(binascii.hexlify(kd(['a', 'b', 'c'])),
                        '0a8ffaf1c23bf0fa6e3a9cc1fa4ff914'+
                        '900213a45cba87d66d9d6c611dcf3cee')
    def test_short(self):
        kd = pol.kd.KeyDerivation.setup({'bits': 256, 'type': 'sha','salt':'c'})
        self.assertEqual(binascii.hexlify(kd(['a', 'b', 'c'], 13)),
                        'b87a32912e780ab8e22555d132')
    def test_double(self):
        kd = pol.kd.KeyDerivation.setup({'bits': 256, 'type': 'sha','salt':'c'})
        self.assertEqual(binascii.hexlify(kd(['a', 'b', 'c'], 64)),
                'b87a32912e780ab8e22555d132fec8c01b2867128ebb4e56dcac029e71ac9'+
                '02f9e6c49cc332427586fef3cd34330d2724494c09044f475b7c47c24774b'+
                '996059')
    def test_long(self):
        kd = pol.kd.KeyDerivation.setup({'bits': 256, 'type': 'sha','salt':'c'})
        self.assertEqual(binascii.hexlify(kd(['a', 'b', 'c'], 128)),
                'b87a32912e780ab8e22555d132fec8c01b2867128ebb4e56dcac029e71ac9'+
                '02f9e6c49cc332427586fef3cd34330d2724494c09044f475b7c47c24774b'+
                '996059a8fe87e36dde9c60b1e3838d5a891d023f58b73667672d3b796224e'+
                '6b7c617bb6b20a9c08b49f40f9b37f5f34be841e957e415638b6cc03cb4c5'+
                '2906044e65e5')


if __name__ == '__main__':
    unittest.main()




""" Code to read psafe3 databases """

import hmac
import uuid
import struct
import hashlib
import logging
import datetime
import cStringIO as StringIO

import mcrypt

l = logging.getLogger(__name__)

TAG = 'PWS3'
EOF = 'PWS3-EOFPWS3-EOF'

class BadPasswordError(ValueError):
    pass

class PSafe3FormatError(ValueError):
    pass

class IntegrityError(ValueError):
    pass

def stretch_key(key, salt, niter):
    h = hashlib.sha256()
    h.update(key)
    h.update(salt)
    H = h.digest()
    for i in xrange(niter):
        H = hashlib.sha256(H).digest()
    return H

def load(f, password):
    l.debug('Reading header ...')
    tag = f.read(4)
    if tag != TAG:
        raise PSafe3FormatError("Tag is wrong.  Is this a PSafe3 file?")
    salt = f.read(32)
    niter = struct.unpack("<I", f.read(4))[0]

    l.debug('Stretching password ...')
    P2 = stretch_key(password, salt, niter)
    HP2 = hashlib.sha256(P2).digest()
    if HP2 != f.read(32):
        raise BadPasswordError

    l.debug('Reading header ...')
    m = mcrypt.MCRYPT('twofish', 'ecb')
    m.init(P2)
    K = m.decrypt(f.read(32))
    L = m.decrypt(f.read(32))
    IV = f.read(16)

    m = mcrypt.MCRYPT('twofish', 'cbc')
    m.init(K, IV)

    l.debug('Decrypting ...')
    plaintext = ''
    hmac_data = ''
    while True:
        b = f.read(16)
        if b == EOF:
            break
        plaintext += m.decrypt(b)

    l.debug('Reading decrypted header ...')
    g = StringIO.StringIO(plaintext)
    in_header = True
    header = {}
    record = {}
    records = []
    had = set()
    while True:
        field = g.read(5)
        if not field:
            break
        length, t = struct.unpack("<IB", field)
        d = g.read(length)
        hmac_data += d
        if t in had:
            l.warn("Field type %s occurs twice", t)
        had.add(t)
        if in_header:
            if t == 0:
                header['version']  = struct.unpack("<H", d)[0]
            elif t == 1:
                header['uuid'] = uuid.UUID(bytes=d)
            elif t == 2:
                header['non-default-preferences'] = d
            elif t == 3:
                header['tree-display-status'] = d
            elif t == 4:
                header['last-save'] = datetime.datetime.fromtimestamp(
                                        struct.unpack("<I", d)[0])
            elif t == 5:
                header['last-save-who'] = d
            elif t == 6:
                header['last-save-what'] = d
            elif t == 7:
                header['last-save-by-user'] = d
            elif t == 8:
                header['last-save-on-host'] = d
            elif t == 9:
                header['database-name'] = d
            elif t == 10:
                header['database-description'] = d
            elif t == 11:
                header['database-filters'] = d
            elif t == 15:
                header['recently-used-filters'] = d
            elif t == 16:
                header['named-password-policies'] = d
            elif t == 17:
                header['empty-groups'] = d
            elif t == 255:
                in_header = False
                had = set()
            else:
                l.warn("Unknown header field: type %s; data %s",
                            t, repr(d))
        else:
            if t == 1:
                record['uuid'] = uuid.UUID(bytes=d)
            elif t == 2:
                record['group'] = d
            elif t == 3:
                record['title'] = d
            elif t == 4:
                record['username'] = d
            elif t == 5:
                record['notes'] = d
            elif t == 6:
                record['password'] = d
            elif t == 7:
                record['creation-time'] = datetime.datetime.fromtimestamp(
                                        struct.unpack("<I", d)[0])
            elif t == 8:
                record['password-modification-time'] = (
                        datetime.datetime.fromtimestamp(
                                        struct.unpack("<I", d)[0]))
            elif t == 9:
                record['last-access-time'] = datetime.datetime.fromtimestamp(
                                        struct.unpack("<I", d)[0])
            elif t == 10:
                record['password-expiry-time'] = (
                        datetime.datetime.fromtimestamp(
                                        struct.unpack("<I", d)[0]))
            elif t == 12:
                record['last-modification-time'] = (
                        datetime.datetime.fromtimestamp(
                                        struct.unpack("<I", d)[0]))
            elif t == 13:
                record['url'] = d
            elif t == 14:
                record['autotype'] = d
            elif t == 15:
                record['password-history'] = d
            elif t == 16:
                record['password-policy'] = d
            elif t == 17:
                record['password-expiry-interval'] = d
            elif t == 18:
                record['run-command'] = d
            elif t == 19:
                record['double-click-action'] = d
            elif t == 20:
                record['email-address'] = d
            elif t == 21:
                record['protected-entry'] = (d != chr(0))
            elif t == 22:
                record['own-symbols-for-password'] = d
            elif t == 23:
                record['shift-double-click-action'] = d
            elif t == 24:
                record['password-policy-name'] = d
            elif t == 255:
                records.append(record)
                record = {}
                had = set()
            else:
                l.warn("Unknown record field: type %s; data %s",
                            t, repr(d))
        tl = length + 5
        if tl % 16 != 0:
            g.read(16 - (tl % 16))
    l.debug('Checking HMAC ...')
    if hmac.new(L, hmac_data, hashlib.sha256).digest() != f.read(32):
        raise IntegrityError
    return (header, records)



import unittest
import tempfile

import pol.main

class TestMain(unittest.TestCase):
    def setUp(self):
        self.safe = tempfile.NamedTemporaryFile()
        self.config = tempfile.NamedTemporaryFile()

    def pol(self, *args):
        ret = pol.main.entrypoint(['-s', self.safe.name,
                                   '-C', self.config.name] + list(args))
        return 0 if ret is None else ret
    def test_basic(self):
        # TODO we should check output
        self.pol('init', '-P', '-p', 'a', 'b', 'c', '-f',
                    '--i-know-its-unsafe', '-N', '128')
        self.assertEqual(self.pol('list', '-p', 'a'), 0)
        self.assertEqual(self.pol('list', '-p', 'b'), 0)
        self.assertEqual(self.pol('put', '-p', 'a', '-s', 'a secret', 'key'), 0)
        self.assertEqual(self.pol('get', '-p', 'a', 'key'), 0)
        self.assertEqual(self.pol('put', '-p', 'b', '-s', 'a secret', 'key'), 0)
        self.assertEqual(self.pol('put', '-p', 'c', '-s', 'a secret', 'key'), 0)
        self.assertEqual(self.pol('put', '-p', 'd', '-s', 'a secret', 'key'), -1)
        self.assertEqual(self.pol('get', '-p', 'a', 'key'), -8)
        self.assertEqual(self.pol('get', '-p', 'b', 'key'), -4)
        self.assertEqual(self.pol('get', '-p', 'c', 'key'), -4)
        self.assertEqual(self.pol('list', '-p', 'a'), 0)
        self.assertEqual(self.pol('list', '-p', 'b'), 0)
        self.assertEqual(self.pol('touch'), 0)
        self.assertEqual(self.pol('export', '-p', 'a'), 0)
    def test_cracktime_names(self):
        self.assertEqual(frozenset(pol.main.cracktime_names),
                         frozenset(pol.main.cracktimes.keys()))


if __name__ == '__main__':
    unittest.main()




""" Implementation of key derivation """

import logging
import hashlib
import struct

import Crypto.Random

l = logging.getLogger(__name__)

class KeyDerivationParameterError(ValueError):
    pass

class KeyDerivation(object):
    """ Cryptographically secure one-way key-derivation """

    def __init__(self, params):
        """ Initialize the KeyDerivation with the given parameters.

            NOTE use KeyDerivation.setup """
        self.params = params

    @staticmethod
    def setup(params=None, randfunc=None):
        """ Set-up the keyderivation given by `params`.
        
            If `params' is None, generates new parameters.  In that case
            `randfunc' is used to generate a salt. """
        if params is None:
            if randfunc is None:
                randfunc = Crypto.Random.new().read
            params = {'type': 'sha',
                      'bits': 256,
                      'salt': randfunc(32)}
        if ('type' not in params or not isinstance(params['type'], basestring)
                or params['type'] not in TYPE_MAP):
            raise KeyDerivationParameterError("Invalid `type' attribute")
        return TYPE_MAP[params['type']](params)

    def __call__(self, args, length=32):
        return self.derive(args, length)

    def derive(self, args, length=32):
        """ Derives a key of `length' bytes from the list of strings `args'. """
        raise NotImplementedError

    @property
    def size(self):
        """ The "natural" size of the key-derivation """
        raise NotImplementedError

class SHAKeyDerivation(KeyDerivation):
    """ SHA is the default key derivation """

    def __init__(self, params):
        super(SHAKeyDerivation, self).__init__(params)
        if not 'salt' in params:
            raise KeyDerivationParameterError("Missing param `salt'")
        if not 'bits' in params:
            raise KeyDerivationParameterError("Missing param `bits'")
        if not isinstance(params['salt'], basestring):
            raise KeyDerivationParameterError("`salt' should be a string")
        if not isinstance(params['bits'], int):
            raise KeyDerivationParameterError("`bits' should be int")
        if params['bits'] not in (256,):
            raise KeyDerivationParameterError(
                    "We do not support the given `bits'")
        self.bits = params['bits']
        self.salt = params['salt']
        self.word_struct = struct.Struct(">H")

    def _derive(self, args):
        """ Derives `self.bits' bit key from args """
        if self.bits == 256:
            new_hash = hashlib.sha256
        else:
            assert False
        oh = new_hash()
        for arg in args:
            oh.update(new_hash(arg).digest())
        return oh.digest()

    def derive(self, args, length=32):
        ret = ''
        byts = self.bits / 8
        n = length / byts
        if length % byts != 0:
            n += 1
        for i in xrange(n):
            ret += self._derive(args + [self.word_struct.pack(i), self.salt])
        return ret[:length]

    @property
    def size(self):
        return self.bits / 8

TYPE_MAP = {'sha': SHAKeyDerivation}



""" Some helper functions for the terminal """

import contextlib
import termios
import logging
import getpass
import select
import string
import fcntl
import sys
import os

import demandimport

with demandimport.ignored('simplejson'):
    import zxcvbn

l = logging.getLogger(__name__)

def prompt_yes_or_no(prompt):
    """ Prompts user for a yes or no """
    sys.stderr.write(prompt)
    sys.stderr.write(' y/n')
    sys.stderr.flush()
    ret = None
    while ret is None:
        c = wait_for_keypress()
        if c == 'y':
            ret = True
        if c == 'n':
            ret = False
    print
    return ret

@contextlib.contextmanager
def raw_mode():
    """ Sets the terminal in a raw mode. """
    flags = fcntl.fcntl(0, fcntl.F_GETFL)
    attrs = termios.tcgetattr(0)
    fcntl.fcntl(0, fcntl.F_SETFL, flags & ~os.O_NONBLOCK)
    new_attrs = list(attrs)
    new_attrs[0] &= ~(termios.IGNBRK | termios.BRKINT | termios.PARMRK
                        | termios.ISTRIP | termios.INLCR | termios.IGNCR
                        | termios.ICRNL | termios.IXON)
    new_attrs[1] &= ~termios.OPOST
    new_attrs[2] &= ~(termios.CSIZE | termios. PARENB)
    new_attrs[2] |= termios.CS8
    new_attrs[3] &= ~(termios.ECHONL | termios.ECHO | termios.ICANON
                        | termios.ISIG | termios.IEXTEN)
    termios.tcsetattr(0, termios.TCSANOW, new_attrs)
    try:
        yield
    finally:
        termios.tcsetattr(0, termios.TCSAFLUSH, attrs)
        fcntl.fcntl(0, fcntl.F_SETFL, flags)

def purge_stdin():
    while len(select.select([0], [], [], 0.0)[0]):
        os.read(0, 4096)

def wait_for_keypress():
    """ Waits for a single keypress """
    with raw_mode():
        purge_stdin()
        try:
            ret = sys.stdin.read(1)
        except KeyboardInterrupt:
            ret = 0
    return ret

def zxcvbn_getpass(prompt, prefix='', allow_empty=True):
    """ Similar to getpass.getpass, but shows password strength while typing """
    pw = None
    current = ''
    checked_strength_of = None
    prompt_offset = len(prefix) + len(prompt)
    purge_stdin()
    sys.stderr.write('\033[1G\033[K')
    sys.stderr.write(prefix + prompt)
    sys.stderr.flush()
    interacted = False
    with raw_mode():
        while True:
            if interacted and not pw and checked_strength_of != current:
                strength = zxcvbn.password_strength(current)
                checked_strength_of = current
                text = '%-4s %3sb %s' % (strength['score'] * '*',
                                int(strength['entropy']),
                                strength['crack_time_display'])
                sys.stderr.write('\033[55G\033[K%s\033[%sG' % (
                            text, prompt_offset + 1))
                sys.stderr.flush()
            c = sys.stdin.read(1)
            interacted = True
            if c == '\r' or c == '\n':
                if not current and not pw:
                    if allow_empty:
                        sys.stderr.write('\033[K\n')
                        sys.stderr.flush()
                        return None
                    sys.stderr.write('\033[1G\033[K')
                    new_prompt = prefix+'No password given.  Retry: '
                    sys.stderr.write(new_prompt)
                    prompt_offset = len(new_prompt)
                    sys.stderr.flush()
                    interacted = False
                    continue
                if pw:
                    if pw != current:
                        sys.stderr.write('\033[1G\033[K')
                        new_prompt = prefix+'Passwords did not match.  Retry: '
                        sys.stderr.write(new_prompt)
                        prompt_offset = len(new_prompt)
                        sys.stderr.flush()
                        pw = None
                        current = ''
                        interacted = False
                        continue
                    sys.stderr.write('\033[K\n')
                    sys.stderr.flush()
                    return pw
                pw = current
                current = ''
                sys.stderr.write('\033[1G\033[K')
                new_prompt = prefix + 'Repeat to verify: '
                sys.stderr.write(new_prompt)
                prompt_offset = len(new_prompt)
                sys.stderr.flush()
            elif c in string.printable:
                current += c
            elif c == '\x17': # C-w
                current = current[:current.rfind(' ', 0, -1)+1]
            elif c == '\x15': # C-u
                current = ''
            elif c == '\x7f': # backspace
                current = current[:-1]
            elif c == '\x03': # C-c
                raise KeyboardInterrupt
            else:
                sys.stderr.write('\033[55G\033[Kignored key %r\033[%sG' % (
                                             c, prompt_offset + 1))
                sys.stderr.flush()



import unittest
import binascii

import pol.blockcipher

class TestAES(unittest.TestCase):
    def setUp(self):
        self.c = pol.blockcipher.BlockCipher.setup()
    def test_default(self):
        self.assertEqual(self.c.params['type'], 'aes')
        self.assertEqual(self.c.params['bits'], 256)
    def test_encrypt(self):
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!')
        self.assertEqual(binascii.hexlify(s.encrypt('1234567890123456')),
                                        '6cef07fd1ac23d4b6f7429871bb99d8e')
        self.assertEqual(binascii.hexlify(s.encrypt('1234567890123456')),
                                        '2f3f54eba982349b503152c6ec848886')
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!')
        self.assertEqual(binascii.hexlify(s.encrypt('1234567890123456'*2)),
                                        '6cef07fd1ac23d4b6f7429871bb99d8e'+
                                        '2f3f54eba982349b503152c6ec848886')
    def test_decrypt(self):
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!')
        self.assertEqual(s.decrypt(binascii.unhexlify(
                            '6cef07fd1ac23d4b6f7429871bb99d8e')),
                            '1234567890123456')
        self.assertEqual(s.decrypt(binascii.unhexlify(
                            '2f3f54eba982349b503152c6ec848886')),
                            '1234567890123456')
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!')
        self.assertEqual(s.decrypt(binascii.unhexlify(
                            '6cef07fd1ac23d4b6f7429871bb99d8e'+
                            '2f3f54eba982349b503152c6ec848886')),
                            '1234567890123456'*2)
    def test_offset(self):
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!')
        p1 = 'qwaszxerdfcvtygh'
        p2 = 'QWASZXERDFCVTYGH'
        p3 = '1234567891234560'
        c1 = s.encrypt(p1)
        c2 = s.encrypt(p2)
        c3 = s.encrypt(p3)
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!')
        self.assertEqual(s.decrypt(c1), p1)
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!',
                                            offset=16)
        self.assertEqual(s.decrypt(c2), p2)
        s = self.c.new_stream('Thirtytwo bytes key for AES-256!',
                                            'Sixteen byte IV!',
                                            offset=32)
        self.assertEqual(s.decrypt(c3), p3)

if __name__ == '__main__':
    unittest.main()




import unittest

import Crypto.Random

import pol.safe

class TestElgamalSafe(unittest.TestCase):
    def test_generate(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True)
    def test_new_slice(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=100)
        safe._new_slice(100)
    def test_new_slice_full(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=100)
        self.assertRaises(pol.safe.SafeFullError, safe._new_slice, 101)
    def test_elgamal(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=1)
        randfunc = Crypto.Random.new().read
        safe._write_block(0, safe._eg_encrypt_block(
                        'key', 0, '123456789', randfunc, annex=True))
        self.assertEqual(safe._eg_decrypt_block('key', 0),
                        '123456789'.ljust(safe.bytes_per_block, '\0'))
        data = randfunc(safe.bytes_per_block)
        safe._write_block(0, safe._eg_encrypt_block(
                        'key', 0, data, randfunc))
        self.assertEqual(safe._eg_decrypt_block('key', 0), data)
    def test_slice_store(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=10)
        sl = safe._new_slice(10)
        self.assertEqual(sl.size, 1224)
        self.assertRaises(pol.safe.WrongKeyError, sl.store, 'key', '!'*sl.size)
        sl.store('key', '!'*sl.size, annex=True)
        sl.store('key', '!'*sl.size)
        self.assertRaises(ValueError, sl.store, 'key', '!'*(sl.size+1))
        sl.store('key', '!'*1)
        sl.store('key', '')
    def test_find_slices(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=10)
        sl1 = safe._new_slice(2)
        sl2 = safe._new_slice(2)
        sl3 = safe._new_slice(2)
        sl1.store('key', '!!!!', annex=True)
        sl2.store('key', '!!!!', annex=True)
        sl3.store('key3', '!!!!', annex=True)
        self.assertFalse(list(safe._find_slices('nokey')))
        self.assertEqual(len(list(safe._find_slices('key'))), 2)
    def test_load_slice(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=10)
        sl = safe._new_slice(5)
        sl.store('key', '!!!!', annex=True)
        self.assertEqual(safe._load_slice('key', sl.first_index).value,
                            '!!!!')
        randfunc = Crypto.Random.new().read
        sl.store('key', 'abcd'*(sl.size/4))
        self.assertEqual(safe._load_slice('key', sl.first_index).value,
                                'abcd'*(sl.size/4))
        data = randfunc(sl.size)
        sl.store('key', data)
        self.assertEqual(safe._load_slice('key', sl.first_index).value, data)
    def test_large_slice(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=70)
        sl = safe._new_slice(70)
        randfunc = Crypto.Random.new().read
        data = randfunc(sl.size)
        sl.store('key', data, annex=True)
        self.assertEqual(safe._load_slice('key', sl.first_index).value, data)
    def test_open_containers(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=70)
        safe.new_container('m', 'l', 'a', nblocks=70)
        cs_m = list(safe.open_containers('m'))
        cs_l = list(safe.open_containers('l'))
        cs_a = list(safe.open_containers('a'))
        self.assertEqual(len(cs_m), 1)
        self.assertEqual(len(cs_l), 1)
        self.assertEqual(len(cs_a), 1)
        c_m = cs_m[0]
        c_l = cs_l[0]
        c_a = cs_a[0]
        self.assertTrue(c_m.can_add)
        self.assertTrue(c_l.can_add)
        self.assertTrue(c_a.can_add)
    def test_additional_keys(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=70)
        safe.new_container('m', 'l', 'a', nblocks=30,
                                additional_keys=['b','a'])
        cs_m = list(safe.open_containers('m'))
        cs_l = list(safe.open_containers('l'))
        cs_a = list(safe.open_containers('a'))
        self.assertEqual(len(cs_m), 0)
        self.assertEqual(len(cs_l), 0)
        self.assertEqual(len(cs_a), 0)
        cs_m = list(safe.open_containers('m', additional_keys=['a', 'b']))
        cs_l = list(safe.open_containers('l', additional_keys=['a', 'b']))
        cs_a = list(safe.open_containers('a', additional_keys=['a', 'b']))
        self.assertEqual(len(cs_m), 1)
        self.assertEqual(len(cs_l), 1)
        self.assertEqual(len(cs_a), 1)
        c_m = cs_m[0]
        c_l = cs_l[0]
        c_a = cs_a[0]
        self.assertTrue(c_m.can_add)
        self.assertTrue(c_l.can_add)
        self.assertTrue(c_a.can_add)
        self.assertEqual(len(list(safe.open_containers('m',
                        additional_keys=['a', 'b', 'c']))), 0)
        self.assertEqual(len(list(safe.open_containers('o',
                        additional_keys=['a', 'b']))), 0)

    def test_main_data(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=70)
        safe.new_container('m', 'l', None, nblocks=70)
        c = list(safe.open_containers('m'))[0]
        self._fill_container(c)
        self._check_container(c)
        c.save()
        c = list(safe.open_containers('m'))[0]
        self._check_container(c)
        self._check_container_secrets(c)
        c = list(safe.open_containers('l'))[0]
        self._check_container(c)
    def test_append_data(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=70)
        safe.new_container('m', 'l', 'a', nblocks=70)
        c = list(safe.open_containers('a'))[0]
        self._fill_container(c)
        self.assertRaises(pol.safe.MissingKey, self._check_container, c)
        c.save()
        c = list(safe.open_containers('m', move_append_entries=False))[0]
        self._check_container(c)
        self._check_container_secrets(c)
        c.save()
        c = list(safe.open_containers('l', move_append_entries=False))[0]
        self.assertEqual(len(list(c.list())), 0)
        c = list(safe.open_containers('m'))[0]
        self._check_container(c)
        self._check_container_secrets(c)
        c.save()
        c = list(safe.open_containers('l', move_append_entries=False))[0]
        self._check_container(c)
    def test_removal(self):
        safe = pol.safe.Safe.generate(precomputed_gp=True, n_blocks=70)
        safe.new_container('m', 'l', 'a', nblocks=70)
        c = list(safe.open_containers('a'))[0]
        self._fill_container(c)
        c.save()
        c = list(safe.open_containers('m', move_append_entries=False))[0]
        list(c.get('key1'))[0].remove()
        list(c.get('key2'))[0].remove()
        c.save()
        c = list(safe.open_containers('m'))[0]
        list(c.get('key3'))[0].remove()
        list(c.get('key4'))[0].remove()
        list(c.get('key4'))[0].remove()
        c.save()
        c = list(safe.open_containers('m'))[0]
        self.assertEqual(len(c.list()), 0)

    def _fill_container(self, c):
        c.add('key1', 'note1', 'secret1')
        c.add('key2', 'note2', 'secret2')
        c.add('key3', 'note3', 'secret3')
        c.add('key4', 'note4', 'secret4')
        c.add('key4', 'note4', 'secret4')
    def _check_container_secrets(self, c):
        self.assertEqual(list(c.get('key1'))[0].secret, 'secret1')
        self.assertEqual(list(c.get('key2'))[0].secret, 'secret2')
        self.assertEqual(list(c.get('key3'))[0].secret, 'secret3')
        self.assertEqual(list(c.get('key4'))[0].secret, 'secret4')
        self.assertEqual(list(c.get('key4'))[1].secret, 'secret4')
    def _check_container(self, c):
        self.assertEqual(len(list(c.get('key1'))), 1)
        self.assertEqual(len(list(c.get('key2'))), 1)
        self.assertEqual(len(list(c.get('key3'))), 1)
        self.assertEqual(len(list(c.get('key4'))), 2)
        self.assertEqual(list(c.get('key1'))[0].key, 'key1')
        self.assertEqual(list(c.get('key2'))[0].key, 'key2')
        self.assertEqual(list(c.get('key3'))[0].key, 'key3')
        self.assertEqual(list(c.get('key4'))[0].key, 'key4')
        self.assertEqual(list(c.get('key4'))[1].key, 'key4')
        self.assertEqual(list(c.get('key1'))[0].note, 'note1')
        self.assertEqual(list(c.get('key2'))[0].note, 'note2')
        self.assertEqual(list(c.get('key3'))[0].note, 'note3')
        self.assertEqual(list(c.get('key4'))[0].note, 'note4')
        self.assertEqual(list(c.get('key4'))[1].note, 'note4')


if __name__ == '__main__':
    unittest.main()




""" Password generation """

import logging
import math

import Crypto.Random.random

import pkg_resources

l = logging.getLogger(__name__)

ALPHABET = {'dense':
                'qwertyuiopasdfghjklzxcvbnm1234567890QWERTYUIOPASDFGHJKLZ'+
                'XCVBNM!@#$%^&*(){}[]-=_+,.<>/?',
            'alphanum':
                'qwertyuiopasdfghjklzxcvbnm1234567890QWERTYUIOPASDFGHJKLZ'+
                'XCVBNM'}
kinds = tuple(ALPHABET.keys()) + ('english', 'dutch')

def generate_password(length=None, entropy=None, kind='dense'):
    add_spaces = False
    if kind == 'english':
        from pol.passgen.english import words as alphabet
        add_spaces = True
    elif kind == 'dutch':
        from pol.passgen.dutch import words as alphabet
        add_spaces = True
    else:
        if kind not in ALPHABET:
            raise ValueError("That `kind' of password is not supported.")
        alphabet = ALPHABET[kind]
    if length and entropy:
        raise ValueError("Only one of `length' and `entropy' "+
                            "should be specified")
    bits = []
    if not (length or entropy):
        entropy = 128
    if not length:
        length = int(math.ceil(entropy / math.log(len(alphabet), 2)))
    for i in xrange(length):
        bits.append(Crypto.Random.random.choice(alphabet))
    if add_spaces:
        return ' '.join(bits)
    return ''.join(bits)



""" Implementation of envelopes  """

import logging

import seccure

import Crypto.Random

l = logging.getLogger(__name__)

class EnvelopeParameterError(ValueError):
    pass

class Envelope(object):
    """ Seals secrets with public/private cryptography """

    def __init__(self, params):
        """ Initialize the Envelope with the given parameters.

            NOTE use Envelope.setup """
        self.params = params

    @staticmethod
    def setup(params=None):
        """ Set-up the Envelope given by `params`. """
        if params is None:
            params = {'type': 'seccure',
                      'curve': 'secp160r1'}
        if ('type' not in params or not isinstance(params['type'], basestring)
                or params['type'] not in TYPE_MAP):
            raise EnvelopeParameterError("Invalid `type' attribute")
        return TYPE_MAP[params['type']](params)

    def generate_keypair(self):
        """ Generates and returns a (public, private)-keypair """
        raise NotImplementedError
    def seal(self, msg, pubkey):
        """ Seals a message `msg' for public key `pubkey'. """
        raise NotImplementedError
    def open(self, ciphertext, privkey):
        """ Opens ciphertext returned by `seal' with the private
            key `privkey' """
        raise NotImplementedError

class SeccureEnvelope(Envelope):
    """ Implementation of Envelope using a modified version of the
        Integrated Encryption Scheme for Elliptic Curves compatible
        with the commandline utility seccure[1].
        
        [1] http://point-at-infinity.org/seccure/ """
    def __init__(self, params):
        super(SeccureEnvelope, self).__init__(params)
        if not 'curve' in params:
            raise EnvelopeParameterError("Missing param `curve'")
        if not isinstance(params['curve'], basestring):
            raise EnvelopeParameterError("`curve' should be string")
        try:
            self.curve = seccure.Curve.by_name(params['curve'])
        except KeyError:
            raise EnvelopeParameterError("Curve %s not supported"
                                % repr(params['curve']))

    def generate_keypair(self, randfunc=None):
        if randfunc is None:
            randfunc = Crypto.Random.new().read
        privkey = randfunc(self.curve.key_bytes)
        pubkey = self.curve.passphrase_to_pubkey(privkey).to_string()
        return (pubkey, privkey)
    def seal(self, msg, pubkey):
        p = self.curve.pubkey_from_string(pubkey)
        return p.encrypt(msg)
    def open(self, ciphertext, privkey):
        p = self.curve.passphrase_to_privkey(privkey)
        return self.curve.decrypt(ciphertext, p)

TYPE_MAP = {'seccure': SeccureEnvelope}



import unittest
import binascii

import pol.progressbar

class TestProgressBar(unittest.TestCase):
    def test_normal(self):
        with pol.progressbar.ProgressBar() as p:
            for i in xrange(100):
                p(i/100.0)
    def test_probablistic(self):
        with pol.progressbar.ProbablisticProgressBar() as p:
            for i in xrange(100):
                p(pol.progressbar.coin(0.005, i))

if __name__ == '__main__':
    unittest.main()




""" Extensions to pycrypto's implementation of ElGamal. """

import math
import time
import logging
import binascii
import threading
import itertools
import collections
import multiprocessing

# pycrypto
import demandimport
import Crypto.Util.number as number

import Crypto.Random

# gmpy
import gmpy

import pol.parallel
import pol.progressbar
import pol.serialization

group_parameters = collections.namedtuple('group_parameters', ('p', 'g'))

l = logging.getLogger(__name__)

# TODO add more explanation
twin_primes_constant = 0.66016181584686961514307768084108829498291015625
asymptotic_safe_prime_density = 2 * twin_primes_constant / math.log(2)

# For debugging it is convenient to use precomputed group parameters.
PRECOMPUTED_GROUP_PARAMS = {
        1025: [
            '737d5bc38a37bac243f563b199fb76ade9f954d7ae0f2343724812681f21f3be1'+
            'fd0c60ce8e05650174b5df9ca68bf039ae9af895137c2b25f875cd3e5a7645ac5'+
            '76ac9dbd5ac1fcc90d65e7524cb7a139f393280504f30af83b5d72a444d691f30'+
            'c2a385e4cfa6b3f18fd31de59b76ebbe5cae2781b438180d21a11839d7baa01',
            '72a7b36dc30de8d0c3807328d103ff04e2cf85978d7ff010fecbbdb98b1df39ce'+
            'a5e7b56dc337d62643bf3d51ad8fe2540344fe5cc7dd20f18afe07e1afe1cb034'+
            'feaffe659c761b4bb55dbf6eb202ffd1c66e7b2170fba4b97dc3053366c12fc71'+
            '39a6ef328d7309aec32bed82885afca3c0188efbb0a4d12b48edd82830c2c01'],
        2049: [
            '1f7b6c8b627fba64eb7811dbf3137f4c02f9f11968173c597d3c0ac8c8824e83d'+
            'eb55c34c409edad265a3f65aabcd665c12fb9b7fbffa052645a43cc2d0b6a1907'+
            'dc192ee7d2acf86157d878d29480c47adadfcfc43f7a779777ffb8e1c674c099b'+
            '20a6251fb4c2c3a597b003f5df0cb7e66ee69490d6379f577051b19e88b642b71'+
            '5fa51e5312fddd8c2b36f136b4b2976baa3a3970956a680c8bf9cf71fe6a60080'+
            '377592ae9b25a99eef0afde788ac2feb0b780daaf7873f14a7efc21596e388f88'+
            '9754480032f0a9faafd800d8b0752ca33033134aceec56cae58b7aab80e793032'+
            '8752d830be2d318edc9026f46031f0171f1cb53c02ab2f4bf9a8e5de401',
            '43dc9cb8ed75d53efe931f7fbb83ace5c0a37631cd5eb8bd3c420fb0bef3cc8a8'+
            'b29c3d59982e1f0e1f32e1a082dad03a44ec9cae3391c42d5b172e688386094c4'+
            'ec167f906c3ccaba9b904f6ec1de9cf176d72ec701a1d6f31c5117428997b1dc0'+
            '067b16bee2b05a91137f903fe10075e27dfb225d34c733d543782806fcf98dfb0'+
            'cf3c17b570e1b639df912d74174936b2dffe71b32ea11dbd14b1d9eb7558a902f'+
            '91c43d595b7e4373eff4bd9e389fe2b5cf833585434b25d9530a36f9d034e0d26'+
            '8254268dc5cddf8010aa2d6196b3d8c0a119f00fbdbff472adc3016f27c5a7374'+
            '3b3c4f643a1990882a3224992cce2fe4c3b418dcb3676f4fdf8a5dade01'],
        4097: [
            '2f3df5a84baadc8725396d3788b5c3e022afe1f31717b4df811eca5fa9e672c68'+
            'e855d34e36d9c5b79ac7d323e3770ff028160bca584e8459b313d9217e17b11f9'+
            'b985a86b8ae04442b152408738e5954d98c52e44393791b503a58c774754ffd14'+
            'e83db62b706fac2472a4eb79c4d0014239247c8e1cecee023c5bf9a59239c5166'+
            '9a22bb9d8a374143323602354be85062615bbb8f1b7a3674cf27472d648f4613b'+
            'd59e0945197648452fd453bb32817dc6b221577996b86b0f04f392516143742b7'+
            '245d926cf9ce154c0a1f45e1ac6d2ef20f64fb030897c489cc95eda047117c240'+
            'e39933d9afad273ff607a4b9f82e507af75dc66d37e2453927d67a8c0217ab243'+
            '28e8808801a06aab963a4574b70b7ae7fabf03271524dee6c70c836047641922f'+
            '1278aba60ce451948891e7e0b3e8fb06a21a10256371dd1c1aac1d95f3d745538'+
            '81f6af2a6dd3a70c1138171ca068af4d83f5952db172c8e6f89e8fba87a3ba241'+
            '8b34079d8053034aedaca662c713ed03dc35ce6e67c4385e09affed4e995e09c1'+
            'd4e7e0e932b4a1fd63ba6ae3920c0cb70af77d67c17f13bc63cb9688463eb6c53'+
            '05a4fcb063ab1ad828523c71af1b75c7f2462c4805d38ed648e5342e19c5c8d87'+
            '0c0e443526a9dbe130a03f8240b1682cb65c1999c8091b5f66217419d2b74d435'+
            '5ebc16900bd7fa2ffe4a566f2ef2bf15123de1669ab56c1eb01',
            'e7264b8afe53f07869ab06261d6a1cec13d34609a2ae47dd18a2b3817eb8ad681'+
            '46face0faebd90d6dcb57de81030d1c34a03308be80d2bfe0c8220088cb1329d7'+
            '9fec8f6ffe6c5dea39f04689a8aeb4c7173d7b159885ac83573e4b0c96fd1b638'+
            'a4e390ea6f8a24dc14d5db3ed22ded2bee234dd3a95156127bfcf56901de3c514'+
            '94dc501561c7af94c73d8fc0bfc8b7f99bb761ed44dc7acbbf5080b2c33b1b734'+
            'dc93832e789db5d4cbb7c6ce78c3b80fa0373145bf88fd05245d896a685fa3028'+
            '445d7239a9a533393f25256d37d5e68006d225a82cfc30b4a83dc17a67241e669'+
            'c37426f725a07a61a4cc1d44f3ffa47c12fd3a55bb624470f0946f2b5f09c7f4e'+
            '97251205f977d50945755b8ac459bd4b00635d3f7766a8443089dc9135d5acc1f'+
            'dddc3fd92c3891e7b455a468c5e721875326b0101864aaed38ecf14e0aebec0b4'+
            '457be9c09bc45139b8eaccafd9d2332f145a9c7d598b5e0e4e67a4632c9ad16e8'+
            'e54823c925d9c8876dc0f92ddaf2587e297fa2da2ed8c3231921d8384f889e210'+
            'ae3e511e0d67b5c4b48a7158f49b29528d32d85120edeef742a9cca5c3444b3b3'+
            '7410e51cb7118b97c9150ebbabc4b5747e364d3b705a2b9ecc01f4ed17f49df7a'+
            'b56d975e3f740c8bc7b77a708bdf86cf55d682c2ccc2afd7b7c57014d3599bac0'+
            '05523817cb087debb711289e28db4fd35da61ad6e4c39106e01']
        }

if not number._fastmath:
    l.warning("pycrypto not built with _fastmath module.  A lot will be quite "+
              "slow")

def _find_safe_prime_initializer(args, kwargs):
    Crypto.Random.atfork()
    kwargs['randfunc'] = Crypto.Random.new().read

def _find_safe_prime(bits, randfunc=None):
    """ Finds a safe prime of `bits` bits """
    r = gmpy.mpz(number.getRandomNBitInteger(bits-1, randfunc))
    q = gmpy.next_prime(r)
    p = 2*q+1
    if gmpy.is_prime(p):
        return p

def precomputed_group_params(bits=1025):
    """ Return precomputed group parameters.

        NOTE For small group parameters this is unsafe. """
    if not bits in PRECOMPUTED_GROUP_PARAMS:
        raise ValueError("No precomputed group parameters of %s bits" % bits)
    p, g = [pol.serialization.string_to_number(binascii.unhexlify(x))
                for x in PRECOMPUTED_GROUP_PARAMS[bits]]
    return group_parameters(p=p, g=g)

def generate_group_params(bits=1025, nworkers=None, use_threads=False,
                                progress=None):
    """ Generates group parameters for ElGamal. """
    # Find a safe prime as modulus.  This will take at least several
    # seconds on a single core.  Thus: we will parallelize.
    start_time = time.time()
    if nworkers is None:
        nworkers = multiprocessing.cpu_count()
    l.debug('Searching for a %s bit safe prime p as modulus on %s workers',
                bits, nworkers)
    safe_prime_density = asymptotic_safe_prime_density / (bits - 1)
    if progress:
        progress('p', None)
        def _progress(n):
            progress('p', pol.progressbar.coin(safe_prime_density, n))
    else:
        _progress = None
    p = pol.parallel.parallel_try(_find_safe_prime, (bits,),
                            initializer=_find_safe_prime_initializer,
                            progress=_progress, nworkers=nworkers,
                            use_threads=use_threads)
    # Find a safe `g` as generator.
    # Algorithm taken from Crypto.PublicKey.ElGamal.generate
    # TODO Should we use a generator of a subgroup for performance?
    if progress:
        progress('g', None)
    l.debug('Searching for a suitable generator g')
    start_time = time.time()
    q = (p - 1) / 2
    while True:
        g = gmpy.mpz(number.getRandomRange(3, p))
        if (pow(g, 2, p) == 1 or pow(g, q, p) == 1 or divmod(p-1, g)[1] == 0):
            continue
        ginv = gmpy.invert(g, p)
        if divmod(p - 1, ginv)[1] == 0:
            continue
        break
    l.debug('Found one in %.2fs', time.time() - start_time)
    return group_parameters(p=p, g=g)

def pubkey_from_privkey(privkey, gp):
    return pow(gp.g, privkey, gp.p)
def string_to_group(s):
    return pol.serialization.string_to_number(s)
def group_to_string(n, size):
    # TODO is mpz.binary() stable?
    return pol.serialization.number_to_string(n).ljust(size, '\0')
def decrypt(c1, c2, privkey, gp, size):
    s = pow(c1, privkey, gp.p)
    invs = gmpy.invert(s, gp.p)
    return group_to_string((invs * c2) % gp.p, size)
def encrypt(string, pubkey, gp, size, randfunc):
    # TODO how small may size be?
    number = string_to_group(string)
    r = string_to_group(randfunc(size))
    c1 = pow(gp.g, r, gp.p)
    s = pow(pubkey, r, gp.p)
    c2 = (number * s) % gp.p
    return (c1, c2)



import logging

import gmpy
import zlib
import msgpack

l = logging.getLogger(__name__)

FMT_MSGPACK         = chr(0)
FMT_ZLIB_MSGPACK    = chr(1)

# TODO is the format of gmpy.mpz.binary() stable?
def string_to_number(s):
    """ Converts a string into a number. """
    return gmpy.mpz(s+'\0', 256)

def number_to_string(number):
    """ Converts a number into a string.
    
    Assumes `number' is a `gmpy.mpz'. """
    tmp = number.binary()
    return tmp[:-1] if tmp[-1] == '\0' else tmp

def son_to_string(obj):
    t = msgpack.dumps(obj)
    tc = zlib.compress(t, 9)
    l.debug("son_to_string: original %s; gzipped: %s", len(t), len(tc))
    if len(tc) < len(t):
        return FMT_ZLIB_MSGPACK + tc
    return FMT_MSGPACK + t

def string_to_son(s):
    if s[0] == FMT_ZLIB_MSGPACK:
        tmp = zlib.decompress(s[1:])
    else:
        assert s[0] == FMT_MSGPACK
        tmp = s[1:]
    return msgpack.loads(tmp, use_list=True)



words = ['abandon', 'abdomen', 'abode', 'about', 'above', 'abroad',
         'abrupt', 'absurd', 'abuse', 'academy', 'accord', 'account',
         'acetate', 'acres', 'across', 'active', 'actor', 'acute', 'adrenal',
         'adult', 'adverse', 'advice', 'adviser', 'affair', 'affairs',
         'affect', 'afraid', 'agency', 'agenda', 'agony', 'agreed', 'ahead',
         'airport', 'airway', 'alarm', 'alarmed', 'album', 'albumin',
         'alcohol', 'alert', 'algae', 'algebra', 'alien', 'alike', 'alive',
         'alkali', 'alleged', 'allied', 'allies', 'allowed', 'alloy', 'alone',
         'alpha', 'altar', 'amateur', 'amino', 'ammonia', 'amount', 'ample',
         'analog', 'analogy', 'analyst', 'anarchy', 'anatomy', 'anchor',
         'anemia', 'angel', 'anger', 'angle', 'angry', 'anguish', 'angular',
         'animal', 'ankle', 'annoyed', 'anomaly', 'another', 'answer',
         'antenna', 'antigen', 'anxiety', 'anybody', 'aorta', 'apart',
         'apology', 'appeal', 'apple', 'apples', 'applied', 'archaic',
         'arena', 'armed', 'armor', 'arousal', 'array', 'arrival', 'arrow',
         'artery', 'article', 'artist', 'ashamed', 'aside', 'asleep', 'aspect',
         'assault', 'assay', 'asset', 'assign', 'assist', 'assumed', 'asthma',
         'asylum', 'atomic', 'atrophy', 'attach', 'attack', 'attempt',
         'audio', 'audit', 'author', 'authors', 'autumn', 'avail', 'avenue',
         'awake', 'award', 'aware', 'awful', 'awkward', 'backup', 'balcony',
         'ballet', 'balloon', 'ballot', 'bamboo', 'banker', 'banner',
         'baptism', 'bargain', 'barley', 'barrel', 'barrels', 'barren',
         'barrier', 'basal', 'bases', 'basic', 'basin', 'basis', 'basket',
         'batch', 'battery', 'battle', 'beach', 'beads', 'beard', 'beast',
         'beaten', 'beauty', 'bedroom', 'behalf', 'behind', 'belief', 'belly',
         'beloved', 'bench', 'benefit', 'benign', 'beyond', 'bicycle',
         'bigger', 'binary', 'biology', 'biopsy', 'bipolar', 'birth', 'bishop',
         'black', 'blade', 'blame', 'blamed', 'blank', 'blanket', 'blast',
         'blend', 'blind', 'bliss', 'block', 'blond', 'blood', 'bloody',
         'bloom', 'blown', 'blues', 'blunt', 'board', 'boards', 'boast',
         'bodily', 'boiler', 'bones', 'bonus', 'boost', 'boots', 'borrow',
         'bosom', 'bother', 'bottle', 'bottom', 'bound', 'bounds', 'bowed',
         'bowel', 'brain', 'brake', 'branch', 'brand', 'brass', 'brave',
         'breach', 'bread', 'breadth', 'break', 'breast', 'breath', 'breed',
         'brick', 'bride', 'bridge', 'brief', 'brigade', 'broad', 'broke',
         'broken', 'broker', 'brother', 'brown', 'browser', 'brush', 'brushed',
         'bucket', 'budget', 'buffalo', 'buffer', 'build', 'built', 'bullet',
         'bunch', 'bundle', 'burden', 'bureau', 'buried', 'burnt', 'burst',
         'button', 'buttons', 'buyer', 'bypass', 'cabin', 'cabinet', 'cache',
         'calcium', 'camera', 'campus', 'canal', 'cancel', 'cancer', 'candle',
         'candy', 'cannon', 'canoe', 'canon', 'canvas', 'capita', 'capsule',
         'captain', 'captive', 'carbon', 'cardiac', 'cards', 'career',
         'careful', 'cargo', 'carotid', 'carpet', 'carrier', 'carry', 'carved',
         'caste', 'castle', 'catalog', 'catch', 'cathode', 'cattle', 'causal',
         'cause', 'cavalry', 'cease', 'cellar', 'census', 'centre', 'century',
         'ceramic', 'certain', 'cheap', 'check', 'cheek', 'cheer', 'cheese',
         'chicken', 'chief', 'chiefly', 'child', 'chill', 'chimney', 'choice',
         'choir', 'chopped', 'chord', 'chorus', 'chose', 'chosen', 'chronic',
         'church', 'cigar', 'cinema', 'circle', 'circuit', 'civic', 'civil',
         'claim', 'clash', 'class', 'classic', 'clause', 'clean', 'clear',
         'clearer', 'cleft', 'clergy', 'clerk', 'clever', 'click', 'cliff',
         'climate', 'climax', 'climb', 'clinic', 'cloak', 'clock', 'close',
         'closer', 'closet', 'cloth', 'clothed', 'clothes', 'cloud', 'coach',
         'coarse', 'coast', 'coffee', 'coffin', 'cohort', 'collar', 'collect',
         'college', 'colon', 'colonel', 'colony', 'color', 'colors', 'colour',
         'colours', 'column', 'combat', 'comedy', 'comes', 'comfort', 'comic',
         'command', 'common', 'compact', 'company', 'compare', 'compass',
         'complex', 'compute', 'concept', 'concern', 'concert', 'conduct',
         'console', 'contact', 'context', 'contour', 'control', 'convert',
         'convex', 'copied', 'copper', 'corner', 'corps', 'corpse', 'corpus',
         'correct', 'corrupt', 'cortex', 'cosmic', 'cosmos', 'costly',
         'costs', 'costume', 'cotton', 'couch', 'cough', 'council', 'counsel',
         'count', 'country', 'county', 'couple', 'course', 'court', 'cousin',
         'cover', 'covers', 'covert', 'crack', 'craft', 'crash', 'cream',
         'creator', 'credit', 'creed', 'creek', 'creep', 'crime', 'crisis',
         'cross', 'crowd', 'crown', 'crude', 'cruel', 'cruelty', 'cruise',
         'crush', 'crushed', 'crust', 'cubic', 'curse', 'curtain', 'curve',
         'curved', 'custody', 'custom', 'customs', 'cycle', 'cyclic',
         'cynical', 'daily', 'dairy', 'damaged', 'damages', 'danger', 'dashed',
         'deadly', 'dealer', 'death', 'debate', 'debris', 'debtor', 'decade',
         'decay', 'decimal', 'decree', 'deeds', 'default', 'defeat', 'defect',
         'defense', 'deficit', 'degree', 'delay', 'delayed', 'della', 'delta',
         'demand', 'demise', 'demon', 'dense', 'deposit', 'depth', 'deputy',
         'derived', 'desert', 'design', 'desire', 'despair', 'despite',
         'destiny', 'detail', 'details', 'device', 'devices', 'devil',
         'devise', 'devoid', 'diagram', 'dialect', 'dialog', 'diamond',
         'diary', 'dictate', 'dietary', 'diffuse', 'digit', 'dilemma',
         'dilute', 'dinner', 'diode', 'dioxide', 'dipole', 'direct', 'dirty',
         'disease', 'disgust', 'dislike', 'display', 'dispose', 'dispute',
         'ditch', 'divers', 'diverse', 'divide', 'divorce', 'doctor', 'dogma',
         'dollar', 'domain', 'donor', 'doomed', 'doorway', 'dorsal', 'doubt',
         'draft', 'dragon', 'drain', 'drama', 'drawer', 'drawn', 'dread',
         'dream', 'dreamed', 'dried', 'drift', 'drill', 'drink', 'drive',
         'driven', 'driver', 'dropped', 'drove', 'drunk', 'drunken', 'dusty',
         'dwell', 'dynamic', 'dynasty', 'eager', 'eagle', 'earlier', 'early',
         'earth', 'earthly', 'easier', 'eastern', 'eaten', 'ecology',
         'economy', 'ecstasy', 'edema', 'edged', 'editor', 'effect', 'effects',
         'effort', 'eighth', 'eighty', 'elbow', 'elderly', 'elect', 'eleven',
         'elite', 'elusive', 'email', 'embassy', 'embrace', 'embryo',
         'empathy', 'emperor', 'empire', 'employ', 'empty', 'enamel',
         'endemic', 'endowed', 'enemy', 'energy', 'engaged', 'enough',
         'entail', 'entire', 'entropy', 'entry', 'episode', 'epoch', 'erect',
         'error', 'escape', 'escaped', 'escort', 'essay', 'estate', 'esteem',
         'eternal', 'ethanol', 'ether', 'ethic', 'ethical', 'ethics', 'ethnic',
         'ethos', 'every', 'exact', 'example', 'excuse', 'exempt', 'exhaust',
         'exhibit', 'exile', 'expense', 'expert', 'exploit', 'export',
         'expose', 'extinct', 'extra', 'extract', 'extreme', 'fabric',
         'facade', 'faced', 'factor', 'factory', 'faculty', 'faint', 'fairy',
         'faith', 'fallen', 'falls', 'false', 'family', 'fancy', 'fantasy',
         'farmer', 'farther', 'father', 'fatigue', 'fatty', 'fault', 'faulty',
         'fauna', 'favor', 'favour', 'fearful', 'feast', 'feather', 'fellow',
         'female', 'fertile', 'fetch', 'fetus', 'feudal', 'fever', 'fewer',
         'fiber', 'fibre', 'field', 'fierce', 'fiery', 'fifteen', 'fifth',
         'fifties', 'fifty', 'final', 'finer', 'finger', 'finish', 'finite',
         'first', 'fiscal', 'fixed', 'flame', 'flank', 'flash', 'flask',
         'flavor', 'fleet', 'flesh', 'float', 'flock', 'flood', 'floor',
         'flora', 'flour', 'flower', 'flown', 'fluid', 'flush', 'flushed',
         'focal', 'focus', 'folks', 'follow', 'folly', 'foolish', 'force',
         'forced', 'foreign', 'forever', 'forged', 'formal', 'format',
         'formed', 'former', 'formula', 'fortune', 'forty', 'forum', 'forward',
         'fossil', 'found', 'fourth', 'fragile', 'frame', 'framed', 'frank',
         'fraud', 'freed', 'freedom', 'fresh', 'friend', 'fringe', 'front',
         'frost', 'fruit', 'fuller', 'funds', 'fungi', 'funny', 'furnace',
         'further', 'futile', 'gallery', 'gamma', 'garden', 'garlic',
         'gastric', 'gather', 'gauge', 'genera', 'generic', 'genesis',
         'genius', 'genome', 'genre', 'gentle', 'gentry', 'genus', 'geology',
         'ghetto', 'ghost', 'given', 'gland', 'glass', 'glasses', 'glimpse',
         'global', 'globe', 'gloom', 'gloomy', 'glory', 'gloves', 'glucose',
         'golden', 'goods', 'gospel', 'gossip', 'great', 'greed', 'green',
         'greet', 'grief', 'groove', 'gross', 'ground', 'grounds', 'group',
         'grouped', 'grown', 'growth', 'guard', 'guide', 'guilt', 'guilty',
         'guinea', 'guitar', 'habit', 'habitat', 'halfway', 'hammer',
         'handful', 'handle', 'hands', 'happier', 'happy', 'harbor', 'harbour',
         'harmful', 'harmony', 'harsh', 'haste', 'hasty', 'haven', 'hedge',
         'heights', 'helpful', 'heresy', 'heroic', 'hidden', 'higher',
         'highway', 'hills', 'history', 'holiday', 'hollow', 'honesty',
         'honey', 'honor', 'honour', 'hopeful', 'hormone', 'horror', 'horse',
         'hostile', 'hotel', 'hours', 'house', 'human', 'humane', 'humans',
         'humor', 'humour', 'hunger', 'hungry', 'hurried', 'hurry', 'husband',
         'hybrid', 'hygiene', 'ideal', 'illegal', 'imagery', 'immense',
         'immune', 'impact', 'impetus', 'implied', 'import', 'improve',
         'impulse', 'income', 'index', 'indices', 'induced', 'inert',
         'inertia', 'infancy', 'influx', 'inform', 'injury', 'inland',
         'inlet', 'innate', 'inner', 'input', 'insane', 'insect', 'insert',
         'inside', 'insulin', 'insult', 'intact', 'intake', 'integer',
         'intense', 'interim', 'intra', 'invalid', 'inverse', 'invite',
         'inward', 'ionic', 'ironic', 'irony', 'island', 'isolate', 'issue',
         'ivory', 'jacket', 'jewelry', 'joint', 'journal', 'journey', 'judge',
         'judges', 'juice', 'jungle', 'junior', 'justice', 'keeper', 'kernel',
         'kidney', 'killer', 'kinase', 'kindly', 'kingdom', 'kinship',
         'kitchen', 'knife', 'knives', 'knock', 'known', 'label', 'labor',
         'labour', 'laden', 'ladies', 'lapse', 'large', 'larger', 'laser',
         'lattice', 'laugh', 'launch', 'laundry', 'lawful', 'lawyer', 'layer',
         'layout', 'leaders', 'league', 'learner', 'lease', 'least', 'leather',
         'leave', 'leaves', 'legacy', 'legal', 'legend', 'lemon', 'length',
         'lengthy', 'lesbian', 'lesser', 'lesson', 'lethal', 'letters',
         'level', 'lever', 'lexical', 'liaison', 'liberty', 'library',
         'license', 'ligand', 'likely', 'limit', 'linear', 'linen', 'links',
         'lipid', 'lithium', 'little', 'lived', 'lively', 'liver', 'lives',
         'loads', 'lobby', 'local', 'locus', 'lodge', 'lodged', 'lofty',
         'logic', 'logical', 'lonely', 'longer', 'loose', 'losses', 'loved',
         'lovely', 'lover', 'lower', 'loyal', 'loyalty', 'lucky', 'lumbar',
         'lumber', 'lumen', 'lunch', 'luxury', 'lymph', 'lyric', 'macro',
         'magic', 'magical', 'magnet', 'maiden', 'majesty', 'major', 'maker',
         'makeup', 'malaria', 'manager', 'mandate', 'manhood', 'mankind',
         'manner', 'manners', 'mantle', 'march', 'margin', 'marker', 'market',
         'married', 'marrow', 'masonry', 'masses', 'massive', 'mastery',
         'match', 'matched', 'matrix', 'maximal', 'maximum', 'mayor', 'meadow',
         'means', 'media', 'median', 'mediate', 'medical', 'medium', 'melody',
         'member', 'memory', 'menace', 'mercury', 'mercy', 'merged', 'merger',
         'merit', 'merry', 'method', 'methyl', 'metric', 'micro', 'middle',
         'midst', 'mighty', 'milieu', 'militia', 'minimal', 'minimum',
         'minor', 'minus', 'minute', 'minutes', 'miracle', 'mirror', 'misery',
         'missile', 'mistake', 'mixed', 'mobile', 'modal', 'model', 'modem',
         'modern', 'modesty', 'module', 'modulus', 'moist', 'molar', 'monarch',
         'money', 'monitor', 'monkey', 'month', 'monthly', 'morale', 'morals',
         'morrow', 'mortar', 'mosaic', 'mother', 'motif', 'motive', 'motor',
         'mould', 'mound', 'mount', 'mouse', 'mouth', 'moved', 'movie',
         'mucosa', 'multi', 'murmur', 'muscle', 'museum', 'music', 'musical',
         'myriad', 'mystery', 'naive', 'named', 'names', 'narrow', 'nasal',
         'nasty', 'native', 'nausea', 'naval', 'nearby', 'nearer', 'needle',
         'neglect', 'negro', 'nephew', 'nerve', 'nerves', 'network', 'neuron',
         'neutron', 'newborn', 'niche', 'nickel', 'niece', 'ninety', 'ninth',
         'nitrate', 'nobody', 'noise', 'noisy', 'nominal', 'normal', 'north',
         'notice', 'noticed', 'notions', 'novel', 'novelty', 'nowhere',
         'nuclear', 'nuclei', 'nucleus', 'number', 'numbers', 'nurse',
         'nursery', 'object', 'obliged', 'ocean', 'ocular', 'offense',
         'offer', 'office', 'officer', 'offset', 'often', 'olive', 'onset',
         'onward', 'opera', 'opium', 'optical', 'optimal', 'optimum', 'orange',
         'orbit', 'orderly', 'organ', 'organic', 'organs', 'origin', 'other',
         'outcome', 'outdoor', 'outlet', 'outlook', 'output', 'outset',
         'outside', 'outward', 'ovarian', 'overall', 'overlap', 'overt',
         'owner', 'oxide', 'oxygen', 'packet', 'pagan', 'painful', 'pains',
         'paint', 'palace', 'palate', 'panel', 'panic', 'pants', 'papal',
         'paper', 'papers', 'parade', 'paradox', 'parcel', 'pardon', 'parish',
         'partner', 'parts', 'party', 'passive', 'paste', 'pastor', 'patch',
         'pathway', 'patrol', 'patron', 'pattern', 'pause', 'paved', 'peace',
         'pelvic', 'pelvis', 'penal', 'penalty', 'pencil', 'penis', 'penny',
         'people', 'pepper', 'peptide', 'perfect', 'peril', 'period', 'permit',
         'person', 'persona', 'persons', 'petty', 'phase', 'phone', 'photo',
         'photon', 'phrase', 'physics', 'piano', 'piece', 'pierced', 'piety',
         'piles', 'pillar', 'pillow', 'pilot', 'pioneer', 'pipes', 'pistol',
         'piston', 'pitch', 'pitched', 'pledge', 'pledged', 'plenty', 'plexus',
         'pocket', 'poetry', 'point', 'poison', 'polar', 'police', 'policy',
         'polite', 'pollen', 'polls', 'polymer', 'poorly', 'popular', 'porch',
         'postwar', 'potato', 'potency', 'pottery', 'poultry', 'pound',
         'poverty', 'power', 'prairie', 'praise', 'prayer', 'precise',
         'preface', 'premise', 'premium', 'pretend', 'pretext', 'pretty',
         'price', 'pride', 'primacy', 'primary', 'prime', 'print', 'prior',
         'prison', 'privacy', 'private', 'pseudo', 'psyche', 'psychic',
         'puberty', 'public', 'pulpit', 'pulse', 'pumps', 'punch', 'pupil',
         'purple', 'purpose', 'purse', 'pursued', 'pursuit', 'pushed',
         'pyramid', 'rabbit', 'racism', 'racist', 'radar', 'radical', 'radio',
         'radius', 'rails', 'railway', 'rainy', 'raise', 'rally', 'ranch',
         'random', 'range', 'ranged', 'rapid', 'ratio', 'reach', 'reactor',
         'ready', 'realism', 'realist', 'realm', 'reason', 'rebel', 'rebuilt',
         'recall', 'receipt', 'recipe', 'record', 'recruit', 'reduced',
         'refer', 'reflex', 'reform', 'refrain', 'refuge', 'refugee',
         'refusal', 'refuse', 'regard', 'regime', 'regimen', 'regret',
         'regular', 'reign', 'reject', 'relaxed', 'relay', 'release', 'relief',
         'remains', 'remark', 'remedy', 'remote', 'removal', 'remove',
         'removed', 'renal', 'renewal', 'renewed', 'repair', 'repeal',
         'repeat', 'reply', 'report', 'reprint', 'rescue', 'rescued',
         'reserve', 'residue', 'resin', 'resist', 'resolve', 'resort',
         'respect', 'respond', 'result', 'resume', 'retail', 'retina',
         'retinal', 'retreat', 'return', 'reveal', 'revenge', 'revenue',
         'reverse', 'review', 'revival', 'revived', 'revolt', 'reward',
         'rhyme', 'rhythm', 'ribbon', 'riches', 'ridden', 'ridge', 'rifle',
         'rights', 'rigid', 'risen', 'risky', 'rival', 'rivalry', 'river',
         'roads', 'robbery', 'robot', 'robust', 'rocket', 'rocky', 'rooms',
         'rotor', 'rough', 'round', 'route', 'royal', 'royalty', 'rubber',
         'rugged', 'ruler', 'rushed', 'saddle', 'safety', 'sailor', 'saint',
         'salad', 'salary', 'sales', 'salmon', 'sample', 'sands', 'sandy',
         'satire', 'sauce', 'saved', 'savings', 'scale', 'scalp', 'scandal',
         'scarce', 'scarlet', 'scene', 'scenery', 'schema', 'scheme',
         'scholar', 'school', 'scope', 'score', 'scores', 'scorn', 'scrap',
         'scratch', 'scream', 'screen', 'screw', 'script', 'search', 'season',
         'seats', 'second', 'secrecy', 'secret', 'sector', 'secular', 'select',
         'selfish', 'seller', 'selves', 'seminar', 'senator', 'senior',
         'sense', 'sensor', 'sensory', 'series', 'sermon', 'serum', 'serve',
         'server', 'service', 'settle', 'setup', 'seven', 'seventh', 'seventy',
         'severe', 'shade', 'shades', 'shadow', 'shaft', 'shake', 'shaken',
         'shallow', 'shame', 'shape', 'shaped', 'share', 'sharp', 'shear',
         'sheath', 'sheep', 'sheer', 'sheet', 'shelf', 'shell', 'shelves',
         'sheriff', 'shield', 'shift', 'shirt', 'shock', 'shoes', 'shook',
         'shoot', 'shore', 'short', 'shout', 'shower', 'shrewd', 'shrink',
         'siege', 'signal', 'silica', 'silicon', 'silly', 'silver', 'similar',
         'simple', 'sincere', 'sinful', 'singer', 'single', 'sinus', 'sixteen',
         'sixth', 'sixties', 'sixty', 'sketch', 'skill', 'skirt', 'skull',
         'slain', 'slave', 'slavery', 'sleep', 'sleeve', 'slice', 'slide',
         'slogan', 'slope', 'small', 'smaller', 'smart', 'smell', 'smile',
         'smoke', 'smooth', 'snake', 'sober', 'society', 'sodium', 'solar',
         'soldier', 'solemn', 'solid', 'solved', 'someone', 'sorrow', 'sorry',
         'sound', 'source', 'south', 'space', 'spaced', 'spare', 'spark',
         'speaker', 'spear', 'species', 'spectra', 'speech', 'speed', 'spell',
         'sperm', 'sphere', 'spinal', 'spindle', 'spirit', 'spirits', 'spite',
         'spleen', 'split', 'spoke', 'spoken', 'sponsor', 'spoon', 'sport',
         'sports', 'spouse', 'spray', 'spread', 'stack', 'staff', 'staged',
         'stain', 'stairs', 'stake', 'stakes', 'stamp', 'stamped', 'stand',
         'stands', 'staple', 'starch', 'stare', 'stark', 'start', 'state',
         'statue', 'status', 'statute', 'stays', 'steady', 'steal', 'steam',
         'steamer', 'steel', 'steep', 'steps', 'sterile', 'stern', 'steroid',
         'stick', 'stiff', 'still', 'stimuli', 'stock', 'stocks', 'stole',
         'stolen', 'stomach', 'stone', 'stool', 'stopped', 'stops', 'store',
         'stores', 'storm', 'story', 'stout', 'stove', 'stuck', 'studied',
         'studio', 'study', 'stuff', 'stuffed', 'stupid', 'style', 'subdued',
         'subject', 'sublime', 'subset', 'subsidy', 'subtle', 'sudden',
         'sugar', 'suicide', 'suite', 'sulfate', 'sulfur', 'sulphur',
         'summary', 'summer', 'summit', 'summons', 'sunny', 'sunset', 'super',
         'superb', 'supper', 'supply', 'support', 'supreme', 'surface',
         'surge', 'surgeon', 'surgery', 'surplus', 'survey', 'suspect',
         'swallow', 'swamp', 'swear', 'sweat', 'sweep', 'sweet', 'swell',
         'swept', 'swift', 'switch', 'swollen', 'sword', 'sworn', 'symbol',
         'symptom', 'syntax', 'system', 'tackle', 'tactics', 'tails', 'taken',
         'tales', 'talks', 'tapped', 'target', 'tariff', 'taste', 'teach',
         'teacher', 'tears', 'teens', 'teeth', 'temper', 'temple', 'tendon',
         'tennis', 'tense', 'tensile', 'tenth', 'terms', 'terrace', 'terrain',
         'terror', 'textile', 'thanks', 'theatre', 'theft', 'theme', 'theorem',
         'theory', 'therapy', 'there', 'thermal', 'thesis', 'thick', 'thief',
         'thigh', 'things', 'think', 'thinker', 'third', 'thirst', 'thirty',
         'thread', 'threads', 'threat', 'three', 'throat', 'throne', 'through',
         'throw', 'thrown', 'thrust', 'thumb', 'thyroid', 'ticket', 'tidal',
         'tiger', 'timber', 'timely', 'times', 'tissue', 'title', 'tobacco',
         'today', 'toilet', 'token', 'tongue', 'tongues', 'tooth', 'topic',
         'topical', 'touch', 'touched', 'tough', 'tourism', 'tourist',
         'toward', 'towel', 'tower', 'toxic', 'toxin', 'treason', 'treat',
         'treaty', 'trench', 'trend', 'tribal', 'tribe', 'tribute', 'trick',
         'tried', 'trigger', 'triple', 'triumph', 'troop', 'troops', 'trout',
         'truck', 'trunk', 'trunks', 'trust', 'trustee', 'truth', 'tubular',
         'tumor', 'tumour', 'tunnel', 'turmoil', 'tutor', 'twelfth', 'twelve',
         'twenty', 'twist', 'twofold', 'typed', 'typical', 'tyranny', 'ulcer',
         'ultra', 'unaware', 'uncle', 'unclear', 'undue', 'uneasy', 'uneven',
         'unfair', 'unhappy', 'unified', 'uniform', 'unitary', 'unite',
         'unjust', 'unknown', 'unlike', 'unseen', 'update', 'upper', 'upset',
         'uptake', 'upward', 'uranium', 'urban', 'urgency', 'urinary',
         'useful', 'uterus', 'utmost', 'vacuum', 'vagina', 'vaginal', 'vague',
         'valid', 'valley', 'value', 'valued', 'values', 'valve', 'vanish',
         'vapor', 'vapour', 'varied', 'variety', 'vault', 'vector', 'vehicle',
         'velvet', 'vendor', 'verbal', 'verdict', 'verge', 'verse', 'vessel',
         'veteran', 'victim', 'victory', 'video', 'viewer', 'viewers',
         'vigor', 'violate', 'violet', 'violin', 'virgin', 'virtue', 'virus',
         'visit', 'visitor', 'vitamin', 'vivid', 'vocal', 'voice', 'voiced',
         'volume', 'vowel', 'vulgar', 'wages', 'wagon', 'waist', 'warfare',
         'warmth', 'warrior', 'wartime', 'washed', 'waste', 'watch', 'waters',
         'waved', 'weakly', 'wealth', 'wealthy', 'weapon', 'weary', 'weather',
         'wedge', 'weeds', 'weekend', 'weekly', 'weigh', 'weird', 'welcome',
         'welfare', 'western', 'whale', 'wheat', 'wheel', 'wheels', 'where',
         'while', 'whisper', 'whistle', 'white', 'whole', 'widow', 'width',
         'window', 'wings', 'winner', 'wisdom', 'witch', 'wives', 'woman',
         'women', 'wooden', 'woods', 'words', 'worker', 'works', 'world',
         'worldly', 'worms', 'worried', 'worry', 'worse', 'worship', 'worst',
         'worth', 'worthy', 'wound', 'woven', 'wrapped', 'wrath', 'wreck',
         'wrist', 'written', 'wrong', 'yearly', 'years', 'yeast', 'yellow',
         'yield', 'young', 'younger', 'youth']



import unittest
import binascii

import pol.envelope

class TestSeccure(unittest.TestCase):
    def setUp(self):
        self.env = pol.envelope.Envelope.setup()
    def test_default(self):
        self.assertEqual(self.env.params['type'], 'seccure')
        self.assertEqual(self.env.params['curve'], 'secp160r1')
    def test_generate_keypair(self):
        self.env.generate_keypair()
    def test_open(self):
        self.assertEqual(self.env.open(binascii.unhexlify(
            '00143617e9c11a7f6b58eca06e2c68b4d098ea4f5bf8fa85aab33721f06a0'+
            'ed4d08bfe7d8ad22bf2ce7507904b3245121df1d88fc691093c77991b3998'),
            'my private key'),
            'This is a very secret message\n')
    def test_roundtrip(self):
        msg = 'these are sekrits'
        pubk, privk = self.env.generate_keypair()
        self.assertEqual(self.env.open(self.env.seal(msg, pubk), privk), msg)

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

import sys
from setuptools import setup
from src.version import get_version

install_requires = [
    'pycrypto >=2.6',        # TODO do we need this version
    'msgpack-python >=0.2',  #      ibidem
    'gmpy >=1.15, <2',       #      ibidem
    'scrypt >=0.5.5, !=0.6.0',#      ibidem
    'yappi >=0.62',          #      ibidem
    'python-mcrypt >=1.1',   #      ibidem
    'lockfile >=0.8',        #      ibidem
    'PyYAML >=3',            #      ibidem
    'zxcvbn >=1.0',
    'seccure >=0.1.2',
    'demandimport >=0.2.1',
        ]

if sys.version_info < (2, 7):
    install_requires.append('argparse >= 0.8')

setup(
    name='pol',
    version=get_version(),
    description='pol, a modern password manager',
    author='Bas Westerbaan',
    author_email='bas@westerbaan.name',
    url='http://github.com/bwesterb/pol/',
    packages=['pol', 'pol.tests', 'pol.importers', 'pol.passgen'],
    package_dir={'pol': 'src'},
    license='GPL 3.0',
    install_requires=install_requires,
    entry_points = {
        'console_scripts': [
                'pol = pol.main:entrypoint',
            ]
        },
    classifiers = [
            'Development Status :: 3 - Alpha',
            'Environment :: Console',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Operating System :: POSIX',
            'Topic :: Security',
        ],
    test_suite='pol.tests',
    ),



""" Read KeePass 1.x databases """

import uuid
import struct
import logging
import hashlib
import binascii
import datetime
import cStringIO as StringIO

# py-crypto
import Crypto.Cipher.AES

class BadPasswordError(ValueError):
    pass

class KeePassFormatError(ValueError):
    pass

SIGNATURE = binascii.unhexlify('03d9a29a65fb4bb5')

FLAG_SHA2       = 1
FLAG_RIJNDAEL   = 2
FLAG_ARC4       = 4
FLAG_TWOFISH    = 8

l = logging.getLogger(__name__)

def unpack_datetime(s):
    # 76543210 76543210 76543210 76543210 76543210
    # yyyyyyyy yyyyyymm mmdddddh hhhhmmmm mmssssss
    if len(s) != 5:
        raise KeePassFormatError("Date/time data must be 5 bytes")
    b = map(ord, s)
    year  =  (b[0]               << 6) | (b[1] >> 2)
    month = ((b[1] & 0b00000011) << 2) | (b[2] >> 6)
    day   =  (b[2] & 0b00111111) >> 1
    hour  =  (b[2] & 0b00000001) << 4  | (b[3] >> 4)
    mins  = ((b[3] & 0b00001111) << 2) | (b[4] >> 6)
    secs  =  (b[4] & 0b00111111)
    return datetime.datetime(year, month, day, hour, mins, secs)

def masterkey_to_finalkey(masterkey, master_seed, master_seed2, key_enc_rounds):
    key = hashlib.sha256(masterkey).digest()
    cipher = Crypto.Cipher.AES.new(master_seed2, Crypto.Cipher.AES.MODE_ECB)
    for r in xrange(key_enc_rounds):
        key = cipher.encrypt(key)
    key = hashlib.sha256(key).digest()
    return hashlib.sha256(master_seed + key).digest()

def load(f, password, keyfile=None):
    if keyfile:
        l.debug('Reading keyfile ...')
        keyfile_bit = keyfile.read()
        if len(keyfile_bit) == 32:
            pass
        elif len(keyfile_bit) == 64:
            keyfile_bit = binascii.unhexlify(keyfile_bit)
        else:
            keyfile_bit = hashlib.sha256(keyfile_bit).digest()
    else:
        keyfile_bit = None

    l.debug('Reading header ...')
    signature = f.read(8)
    if signature != SIGNATURE:
        raise KeePassFormatError('Invalid signature.  Is this a KeePass file?')
    flags, version = struct.unpack('<II', f.read(8))
    master_seed = f.read(16)
    encryption_iv = f.read(16)
    ngroups, nentries  = struct.unpack('<II', f.read(8))
    contents_hash = f.read(32)
    master_seed2 = f.read(32)
    key_enc_rounds = struct.unpack('<I', f.read(4))[0]

    if flags != FLAG_SHA2 | FLAG_RIJNDAEL:
        raise NotImplementedError

    l.debug('Deriving finalkey ...')
    if keyfile_bit:
        compositekey = hashlib.sha256(password).digest() + keyfile_bit
    else:
        compositekey = password
    finalkey = masterkey_to_finalkey(compositekey, master_seed, master_seed2,
                        key_enc_rounds)

    l.debug('Reading remaining ciphertext ...')
    ciphertext = f.read()

    l.debug('Decrypting ...')
    cipher = Crypto.Cipher.AES.new(finalkey, Crypto.Cipher.AES.MODE_CBC,
                                            encryption_iv)
    padded_plaintext = cipher.decrypt(ciphertext)
    plaintext = padded_plaintext[:-ord(padded_plaintext[-1])]

    l.debug('Verifying hash ...')
    if hashlib.sha256(plaintext).digest() != contents_hash:
        raise BadPasswordError

    l.debug('Parsing groups ...')
    groups_found = 0
    g = StringIO.StringIO(plaintext)
    groups = {}
    current_group = {}
    had = set()
    while groups_found < ngroups:
        field_type, field_size = struct.unpack('<HI', g.read(6))
        if field_type in had:
            raise KeePassFormatError("Same field type occurs twice")
        had.add(field_type)
        data = g.read(field_size)
        if field_type == 0:
            l.debug(' comment %s', field_type, repr(data))
        elif field_type == 1:
            if len(data) != 4:
                raise KeePassFormatError("Group ID data must be 4 bytes")
            value = struct.unpack('<I', data)[0]
            current_group['id'] = value
            l.debug(' id %s', value)
        elif field_type == 2:
            value = data[:-1].decode('utf-8')
            current_group['name'] = value
            l.debug(' name %s', value)
        elif field_type == 3:
            value = unpack_datetime(data)
            current_group['creation-time'] = value
            l.debug(' creation-time %s', value)
        elif field_type == 4:
            value = unpack_datetime(data)
            current_group['last-modification-time'] = value
            l.debug(' last-modification-time %s', value)
        elif field_type == 5:
            value = unpack_datetime(data)
            current_group['last-access-time'] = value
            l.debug(' last-access-time %s', value)
        elif field_type == 6:
            value = unpack_datetime(data)
            current_group['expiration-time'] = value
            l.debug(' expiration-time %s', value)
        elif field_type == 7:
            if len(data) != 4:
                raise KeePassFormatError("Image ID data must be 4 bytes")
            value = struct.unpack('<I', data)[0]
            current_group['image-id'] = value
            l.debug(' image-id %s', value)
        elif field_type == 8:
            if len(data) != 2:
                raise KeePassFormatError("Level data must be 2 bytes")
            value = struct.unpack('<H', data)[0]
            current_group['level'] = value
            l.debug(' level %s', value)
        elif field_type == 9:
            if len(data) != 4:
                raise KeePassFormatError("Flags data must be 2 bytes")
            value = struct.unpack('<I', data)[0]
            current_group['flags'] = value
            l.debug(' flags %s', bin(value))
        elif field_type == 0xffff:
            l.debug(' end-of-group')
            groups_found += 1
            groups[current_group['id']] = current_group
            had = set()
            current_group = {}
        else:
            l.warn(' unknown field %s %s', field_type, repr(data))

    l.debug('Parsing entries ...')
    entries_found = 0
    entries = []
    current_entry = {}
    had = set()
    while entries_found < nentries:
        field_type, field_size = struct.unpack('<HI', g.read(6))
        if field_type in had:
            raise KeePassFormatError("Same field type occurs twice")
        had.add(field_type)
        data = g.read(field_size)
        if field_type == 0:
            l.debug(' comment %s', field_type, repr(data))
        elif field_type == 1:
            if len(data) != 16:
                raise KeePassFormatError("UUID data must be 16 bytes")
            value = uuid.UUID(bytes=data)
            current_entry['uuid'] = value
            l.debug(' uuid %s', value)
        elif field_type == 2:
            if len(data) != 4:
                raise KeePassFormatError("Group ID data must be 16 bytes")
            value = struct.unpack("<I", data)[0]
            current_entry['group'] = value
            l.debug(' group %s', value)
        elif field_type == 3:
            if len(data) != 4:
                raise KeePassFormatError("Image ID data must be 16 bytes")
            value = struct.unpack("<I", data)[0]
            current_entry['image-id'] = value
            l.debug(' image-id %s', value)
        elif field_type == 4:
            value = data[:-1].decode('utf-8')
            current_entry['title'] = value
            l.debug(' title %s', value)
        elif field_type == 5:
            value = data[:-1].decode('utf-8')
            current_entry['url'] = value
            l.debug(' url %s', value)
        elif field_type == 6:
            value = data[:-1].decode('utf-8')
            current_entry['username'] = value
            l.debug(' username %s', value)
        elif field_type == 7:
            value = data[:-1].decode('utf-8')
            current_entry['password'] = value
            l.debug(' password %s', value)
        elif field_type == 8:
            value = data[:-1].decode('utf-8')
            current_entry['notes'] = value
            l.debug(' notes %s', value)
        elif field_type == 9:
            value = unpack_datetime(data)
            current_entry['creation-time'] = value
            l.debug(' creation-time %s', value)
        elif field_type == 10:
            value = unpack_datetime(data)
            current_entry['last-modification-time'] = value
            l.debug(' last-modification-time %s', value)
        elif field_type == 11:
            value = unpack_datetime(data)
            current_entry['last-access-time'] = value
            l.debug(' last-access-time %s', value)
        elif field_type == 12:
            value = unpack_datetime(data)
            current_entry['expiration-time'] = value
            l.debug(' expiration-time %s', value)
        elif field_type == 13:
            value = data[:-1].decode('utf-8')
            current_entry['binary-description'] = value
            l.debug(' binary-description %s', value)
        elif field_type == 14:
            value = data
            current_entry['binary-data'] = value
            l.debug(' binary-data %s', repr(value))
        elif field_type == 0xffff:
            l.debug(' end-of-entry')
            entries_found += 1
            entries.append(current_entry)
            had = set()
            current_entry = {}
        else:
            l.warn(' unknown field %s %s', field_type, repr(data))
    return (groups, entries)



""" Speed measurements of components of pol """

import timeit
import functools

import pol.kd
import pol.ks
import pol.elgamal
import pol.envelope
import pol.blockcipher

import Crypto.Random

def main(program):
    data = []
    kd = pol.kd.KeyDerivation.setup()
    data.append(('kd.derive (1000x)',
            timeit.repeat(functools.partial(kd.derive, ['']),
                                repeat=3, number=1000)))

    ks = pol.ks.KeyStretching.setup()
    data.append(('ks.stretch', timeit.repeat(functools.partial(ks.stretch, ''),
                                repeat=3, number=1)))

    bs = pol.blockcipher.BlockCipher.setup()
    def bs_encrypt():
        s = bs.new_stream('!'*32, '!'*16)
        s.encrypt(' '*20480)
    data.append(('blockcipher encrypt (500x 20KB)', timeit.repeat(bs_encrypt,
                                repeat=3, number=500)))

    def bs_decrypt():
        s = bs.new_stream('!'*32, '!'*16)
        s.decrypt(' '*20480)
    data.append(('blockcipher decrypt (500x 20KB)', timeit.repeat(bs_decrypt,
                                repeat=3, number=500)))

    randfunc = Crypto.Random.new().read
    data.append(('random (1000x 64B)',
            timeit.repeat(functools.partial(randfunc, 64),
                                repeat=3, number=1000)))

    data.append(('random (5x 1MB)',
            timeit.repeat(functools.partial(randfunc, 1024*1024),
                                repeat=3, number=5)))

    gp = pol.elgamal.precomputed_group_params()
    privkey = pol.elgamal.string_to_group(kd([], length=128))
    pubkey = pol.elgamal.pubkey_from_privkey(privkey, gp)
    c1, c2 = pol.elgamal.encrypt('!'*128, pubkey, gp, 128, randfunc)
    data.append(('EG pubkey_from_privkey (100x)',
            timeit.repeat(functools.partial(pol.elgamal.pubkey_from_privkey,
                            privkey, gp), repeat=3, number=100)))
    data.append(('EG encrypt (100x)',
            timeit.repeat(functools.partial(pol.elgamal.encrypt,
                                '!'*128, pubkey, gp, 128, randfunc),
                            repeat=3, number=100)))
    data.append(('EG decrypt (100x)',
            timeit.repeat(functools.partial(pol.elgamal.decrypt,
                                c1, c2, privkey, gp, 128),
                            repeat=3, number=100)))

    data.append(('string_to_number (10000x)',
            timeit.repeat(functools.partial(pol.serialization.string_to_number,
                            '!'*128), repeat=3, number=10000)))

    number = pol.serialization.string_to_number('!'*128)
    data.append(('number_to_string (10000x)',
            timeit.repeat(functools.partial(pol.serialization.number_to_string,
                            number), repeat=3, number=10000)))

    data.append(('_find_safe_prime',
            timeit.repeat(functools.partial(pol.elgamal._find_safe_prime,
                            1025, randfunc), repeat=3, number=1)))

    envelope = pol.envelope.Envelope.setup()
    data.append(('envelope gen. keypair (50x)',
            timeit.repeat(envelope.generate_keypair,
                            repeat=3, number=50)))
    pubkey, privkey = envelope.generate_keypair()
    msg = envelope.seal('!', pubkey)
    data.append(('envelope seal (50x)',
            timeit.repeat(functools.partial(envelope.seal, '!'*128, pubkey), 
                            repeat=3, number=50)))
    data.append(('envelope open (50x)',
            timeit.repeat(functools.partial(envelope.open, msg, privkey), 
                            repeat=3, number=50)))



    for desc, res in data:
        print '%-40s %.4f %.4f %.4f' % (desc, res[0], res[1], res[2])



if __name__ == '__main__':
    main(None)
